// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This is a template for an own actor type
// insert stuff at all locations that have [ xxxxx ]
// by default, this file cannot be compiled.
// $Id: FTATagMon.java,v 1.3 2001/06/21 14:57:09 veit Exp $

package jfta.actors;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

import java.util.*;

/**
  * This actor displays the tag types that are on a tagboard
  * @author Holger Veit
  * @version $Id: FTATagMon.java,v 1.3 2001/06/21 14:57:09 veit Exp $
  */
public class FTATagMon extends FTAActor implements FTAActorI {

  /**
    */
  private transient long lastSeen = -1;
  private transient boolean full = true;
  private transient String tbname;  

  /**
    * eval method
    * @return FTAActor.TERMINATE if the actor terminates after this
    *   routine has been called, FTAActor.KEEPALIVE to have the actor
    *   be rescheduled in the next tick cycle.
    * @exception FTAEvalActorException thrown in case a fatal error occurred
    */
  public int eval() throws FTAEvalActorException {
    try {
      long time = getTagboardTime(tbname,RELATIVE);
      if (time == lastSeen)
	return KEEPALIVE;
      lastSeen = time;

      String[] types = getTagTypes(tbname);
      int i;
      if (types.length == 0) {
        if (full) {
          userMessage("FTATagMon",tbname+" is empty");
	  full = false;
	}
      } else {
	for (i=0; i<types.length; i++) {
	  String type = types[i];
	  int cnt = getTagCount(tbname,type);
	  userMessage("FTATagMon",tbname + ": " + type +
		      " (" + cnt + " tag(s))");
	}
	full = true;
      }

      return FTAActor.KEEPALIVE;
    } catch (FTAException e) {
      throw new FTAEvalActorException("Failed to access tagboard");
    }
  }
  
  /**
    * init method
    * @param args list of arguments passed by <code>addActor</code> to the
    *   newly created actor. 
    * @exception FTAArgumentException to be thrown by the user code if some
    *   argument is incoreect for this type of actor
    * @return returns true, if the initialization was okay, false if there
    *   has been a problem not requiring an <code>FTAArgumentException</code>
    */
  public void init(String[] args) throws FTAArgumentException {
    if (args.length < 1)
      throw new FTAArgumentException("FTATagMon: requires tbname");
    tbname = args[0];
  }
  
  /**
    */
  public FTATagMon() {
    super();
  }
}
