// $Id: FTAMsg.java,v 1.14 2001/06/21 14:57:10 veit Exp $

// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.


package jfta.base;

import java.io.*;
import java.net.*;
import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

public class FTAMsg implements Serializable {

    protected final static FTAService _svc = FTAService.Instance();
    public final static int DATASIZE = 4096;
    public final static int FTAMAGIC = 0x46544130;
    public final static int CMDSIZE = 512;

    public final static int MSGINVALID = 0;

    public final static int CREATEHOST = 1;
    public final static int CREATEHOSTDONE = 2;
    public final static int CREATETB = 3;
    public final static int CREATETBDONE = 4;
    public final static int CREATEEN = 5;
    public final static int CREATEENDONE = 6;
    public final static int FLIPORDERREQ = 7;
    public final static int CONTROL = 8;
    public final static int REMOVEHOST = 9;
    public final static int REMOVETB = 10;
    public final static int REMOVEEN = 11;
    public final static int STARTCYCLE = 12;
    public final static int ACKCYCLE = 13;
    // 14 unused
    // 15 unused
    public final static int FLIPORDER = 16;
    public final static int CYCLE = 17;
    public final static int PREFLIP = 18;
    public final static int CONSENT = 19;
    public final static int POSTFLIP = 20;
    public final static int ADDETSYNC = 21;
    // 22 unused
    public final static int REMOVEETSYNC = 23;
    // 24 unused
    // 25 unused
    // 26 unused
    public final static int PUTTAG = 27;
    public final static int ADDACTOR = 28;
    // 29 unused
    public final static int FTAD_DAEMONREQ = 30;
    public final static int FTAD_DAEMONREPLY = 31;
    public final static int FTAD_SYSTEMREQ = 32;
    public final static int FTAD_SYSTEMREPLY = 33;
    // 34 unused
    // 35 unused
    public final static int FTAD_DIRREQ = 36;
    public final static int FTAD_DIRREPLY = 37;
    public final static int FTAD_SHUTDOWN = 38;
    public final static int LOG_MESSAGE = 39;
    public final static int CREATEENOFFICEDONE = 40;
    public final static int CREATETBOFFICEDONE = 41;
    public final static int ADDHOST = 42;
    public final static int MAX_MESSAGE = 43;

    public final static String getTypeNum(int num) {
	return _dispatch[num].getTypeName();
    }

    private final static int MAXUDPSIZE = 65507;

    protected transient FTAMsg _next;

    public String getTypeName() {
	return "FTAMsg";
    }

    /**
     * discard a message no longer needed
     */
    public void discard() {}

    protected FTAMsg() {}

    public int isA() {
	return 0;
    }

    /**
     * serializer
     */
    public byte[] pack(Object o) throws IOException {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	ObjectOutputStream oos = new ObjectOutputStream(baos);
	oos.writeObject(o);
	oos.flush();
	byte[] b = baos.toByteArray();
	int sz = baos.size();
	if (sz > MAXUDPSIZE) {
	    throw new IOException("Message too long: sz="+sz);
	}
	oos.close();
	return b;
    }

    /**
     * deserializer
     */
    public final static Object unpack(byte[] b) 
	throws IOException,ClassNotFoundException {
	ByteArrayInputStream bais = new ByteArrayInputStream(b);
	ObjectInputStream ois = new ObjectInputStream(bais);
	Object obj = ois.readObject();
	ois.close();
	return obj;
    }

    /**
     * dispatcher array
     */
    private final static FTAMsg[] _dispatch = {
	new FTAMsg(),
	new FTACreateHostMsg(),
	new FTACreateHostDoneMsg(),
	new FTACreateTBMsg(),
	new FTACreateTBDoneMsg(),
	new FTACreateENMsg(),
	new FTACreateENDoneMsg(),
	new FTAFlipOrderReqMsg(),
	new FTAControlMsg(),
	new FTARemHostMsg(),
	new FTARemTBMsg(),
	new FTARemENMsg(),
	new FTAStartCycleMsg(),
	new FTAAckCycleMsg(),
	null, // FLIPNOW
	null, // FLIPDONE
	null, // FLIPORDER
	null, // CYCLE
	null, // PREFLIP
	null, // CONSENT
	null, // POSTFLIP
	new FTAAddETSyncMsg(),
	null, // 22 unused
	new FTARemETSyncMsg(),
	null, // 24 unused
	null, // 25 unused
	null, // 26 unused
	new FTAPutTagMsg(),
	new FTAAddActorMsg(),
	null, // 29 unused
	new FTADaemonReqMsg(),
	new FTADaemonReplyMsg(),
	new FTASystemReqMsg(),
	new FTASystemReplyMsg(),
	null, // 34 unused
	null, // 35 unused
	new FTADirReqMsg(),
	new FTADirReplyMsg(),
	new FTAShutdownMsg(),
	new FTALogMsg(),
	new FTACreateENOfficeDoneMsg(),
	new FTACreateTBOfficeDoneMsg(),
	new FTAAddHostMsg(),
    };

    /**
     * reader dispatcher.
     * This reads a the message header and then, if there is
     * a registered message handler, calls the appropriate
     * specific reader. If none is matched, the slow object
     * reader is used.
     */
    public static FTAMsg read(DataInputStream in,byte[] helpbuf)
	throws IOException {
	int magic = in.readInt();
	if (magic == FTAMAGIC) {
	    short type = in.readShort();
	    FTAMsg rdr = _dispatch[type];
	    if (rdr == null) 
		rdr = _dispatch[0];
	    return rdr.readMsg(in,helpbuf);
	} else {
	    FTADebug.fatal("Bad magic in FTAMsg.read: "+magic);
	}
	return null;
    }

    /**
     * default writer (slow)
     *
     * The default writer packs a message into a byte array
     * filled by an object output stream and sends this
     * byte array.
     * This is called by an FTA*ClientChannel for a message
     * to be sent out a DataOutputStream.
     * A specific message type may overwrite this method with
     * a faster one.
     */
    public void write(DataOutputStream out) throws IOException {
	try {
	    out.writeInt(FTAMAGIC);
	    out.writeShort((short)isA());
	    writeMsg(out);
	    out.flush();
	}
	catch (IOException e) {
	    throw new IOException(e.getMessage());
	}
	catch(Exception e) {
	    e.printStackTrace();
	    throw new IOException(e.getMessage());
	}
    }

    /**
     * This is a specific reader method for a message
     * register an empty instance of that message type
     * in _dispatch[] to get this called
     * By default, this uses the slow object unpacker
     */
    protected FTAMsg readMsg(DataInputStream in,
			     byte[] helpbuf) throws IOException {
	synchronized(helpbuf) {
	    int len = in.readInt();
	    in.readFully(helpbuf,0,len);
	    try {
		return (FTAMsg)unpack(helpbuf);
	    } catch (ClassNotFoundException e) {
		FTADebug.fatal("Internal error in FTAMsg");
		return null;
	    }
	}
    }

    /**
     * improved version of DataOutputStream.writeUTF
     */
    protected void writeString(DataOutputStream out,String str) 
	throws IOException {
	int len = str.length();
	out.writeShort((short)len);
	out.writeChars(str);
    }

    /**
     * improved version of DataInputStream.readUTF
     */
    protected String readString(DataInputStream in,byte[] helpbuf) 
	throws IOException {
	int len = in.readShort();
	StringBuffer sb = new StringBuffer(len);
	for (int i=0; i<len; i++)
	    sb.append(in.readChar());
	return sb.toString();
    }


    /**
     * This is a specific writer method for a message.
     * Register an empty instance of that message type
     * in _dispatch[] to get this called.
     * By default, this code will use the slow object packer
     */
    protected void writeMsg(DataOutputStream out) 
	throws IOException {
	byte[] msgbuf = pack(this);
	int len = msgbuf.length;
	out.writeInt(len);
	out.write(msgbuf,0,len);
    }

    public String toString() {
	return "FTAMsg";
    }

    public void processListener(FTAListenerList list) {
	FTADebug.fatal("Attempt to process baseclass");
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
