// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAPropertyFile.java,v 1.2 2001/06/21 14:57:10 veit Exp $

package jfta.base;

import java.util.*;
import java.io.*;

import jfta.http.*;

/**
 * This is a persistent store for key/value pairs
 */
public class FTAPropertyFile {
    private File _file;
    private Properties _prop;

    /**
     * Create a new property, and read in properties from file.
     * If the file does not exist, the property list starts empty.
     * @param filename name of file to load initially
     */
    public FTAPropertyFile(String filename) {
	_prop = new Properties();
	_file = new File(filename);
	load();
    }

    /**
     * add a property to the persistent store. Will save the
     * new state.
     * @param key key of the property
     * @param value value of the property
     */
    public void add(String key,String value) {
	_prop.setProperty(key,value);
	save();
    }

    /**
     * remove a property from the repositiory.
     * @param key key of the property
     */
    public void remove(String key) {
	_prop.remove(key);
	save();
    }

    /**
     * Return an iterator over all stored keys
     * @return iterator
     */
    public Enumeration keys() {
	return _prop.propertyNames();
    }

    /**
     * get a property from the repository.
     * @param key key of the property
     */
    public String get(String key) {
	return _prop.getProperty(key);
    }

    /**
     * return true if there exists a proprty with the given key.
     * new state.
     * @param key key of the property
     * @return true if property exists, false if not.
     */
    public boolean contains(String key) {
	return _prop.containsKey(key);
    }

    /**
     * clear the whole repository. The file backing the data will
     * be empty afterwards.
     */
    public void clear() {
	_prop.clear();
	save();
    }

    private void load() {
	try {
	    _prop.load(new FileInputStream(_file));
	}
	catch (IOException e) {
	    // ignore that - simply is not existing so far
	}
    }

    private void save() {
	try {
	    _prop.store(new FileOutputStream(_file),
			"This file is automatically generated, don't edit!");
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
    }

    /**
     * build a select box for HTML
     */
    public void selectBox(FTAHtmlFile f,String name,
			  String first,String selected) {
	f.selectBox(name,_prop,first,selected);
    }
}
