// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTAService.java,v 1.8 2001/06/21 14:57:10 veit Exp $

package jfta.base;

import java.net.*;
import java.util.*;
import java.io.*;
import java.awt.*;

import jfta.bootstrap.*;
import jfta.exception.*;

/**
 * The FTA Service class provides multiple general services for FTA
 */
public class FTAService {

    /**
     * <code>FTADIRSERVER</code> points to the public URL of the DirServer
     */
    public static String FTADIRSERVER;
    /**
     * <code>FTADAEMONPORT</code> contains the public port number
     * this daemon is using
     */
    public static int FTADAEMONPORT;
    /**
     * <code>FTADAEMONADDRPOOL</code> contains the address prefix for
     * Multicast addresses
     */
    public static String FTADAEMONADDRPOOL;
    /**
     * <code>FTADAEMONMULTIADDR</code> contains the Multicastaddress
     * the daemons of this cluster use
     */
    public static InetAddress FTADAEMONMULTIADDR;
    /**
     * <code>FTADAEMONTTL</code> is the time-to-live for multicast packets
     *
     */
    public static int FTADAEMONTTL;

    private Hashtable _daemons;
    private Hashtable _systems;
    private Hashtable _multicasts;
    private Properties _userprop;
    private static FTAService _instance = null;
    private String _hostname = null;
    private String _domainname = null;
    private boolean rmiwait = false;
    private FTADaemon _mydaemon;

    /**
     * Creates a new <code>FTAService</code> instance.
     *
     * @roseuid 38B3D55334E8
     * protected constructor, cannot be used (Singleton pattern)
     */
    protected FTAService() {
	super();
	_daemons = new Hashtable();
	_systems = new Hashtable();
	_userprop = new Properties();
	_multicasts = new Hashtable();

	try {
	    // read the user properties
	    FileInputStream fis =
		new FileInputStream(System.getProperty("user.home") +
				    File.separator + "jfta.properties");
	    _userprop.load(fis);
	    fis.close();
	    
	    FTADIRSERVER = getProperty("fta_directory_server_host",
				       "localhost");
	    FTADAEMONPORT = getIntegerProperty("fta_daemon_port",6164);
	    FTADAEMONADDRPOOL = getProperty("fta_multicast_addrpool","224.77.");
	    FTADAEMONMULTIADDR = InetAddress.getByName(FTADAEMONADDRPOOL+"0.0");
	    FTADAEMONTTL = getIntegerProperty("fta_daemon_ttl",32);
	}
	catch (IOException e) {
	    FTADebug.fatal("No ~/jfta.properties file. jFTA won't run");
	}

	_hostname = getHostname();
	_userprop.put("hostname",_hostname);
	_userprop.put("fullname",getFQDNname(_hostname));
    }
    
    /**
     * Construct the Singleton instance of this object
     * @roseuid 38B3D55381A0
     * @return the instance
     */
    public static FTAService Instance() {
	if (_instance == null) {
	    _instance = new FTAService();
	}
	return _instance;
    }

    /**
     * return the fully qualified domain name of this host
     * @return the hostname
     * @roseuid 38B3D553E150
     */
    public String getFQDNname() {
	return getFQDNname(_hostname);
    }
  
    /**
     * return the fully qualified domain name for a given host
     * @param hostname a hostname with or without domain extension
     * @return returns the fully qualified domain name
     * @roseuid 38B3D553E151
     */
    public String getFQDNname(String hostname) {
	if (hostname==null)
	    return getFQDNname();
	if (hostname.indexOf('.') != -1)
	    return hostname;
	return new String(hostname + "." + getDomainname());
    }
  
    /**
     * return the domain of this system
     * @return the domainname
     * @roseuid 38B3D553E153
     */
    private String getDomainname() {
	if (_domainname == null) {
	    // XXX should obtain the resource from somewhere
	    _domainname = getProperty("domainname","gmd.de");
	}
	return _domainname;
    }
  
    /**
     * get an FTA property. This will first search the system properties
     * and then the user property file at ~/jfta.properties
     * @param key property name
     * @param alter alternative property in case the key was not matched
     * @return the property value
     * @roseuid 38B3D553E156
     */
    public String getProperty(String key, String alter) {
	String prop = _userprop.getProperty(key);
	if (prop == null) {
	    try {
		prop = System.getProperty(key);
		if (prop == null)
		    return alter;
	    }
	    catch(SecurityException e) {
		return alter;
	    }
	}
	return prop;
    }

    /**
     * get a boolean FTA property.
     * @param key property name
     * @param alter alternative property in case the key was not matched
     * @return the property value
     * @roseuid 38B3D553E156
     */
    public boolean getBooleanProperty(String key, boolean alter) {
	String prop = getProperty(key,null);
	if (prop==null)
	    return alter;
	return Boolean.valueOf(prop).booleanValue();
    }

    /**
     * get an integer FTA property.
     * @param key property name
     * @param alter alternative property in case the key was not matched
     * @return the property value
     */
    public int getIntegerProperty(String key, int alter) {
	String prop = getProperty(key,null);
	if (prop==null)
	    return alter;
	return atoi(prop,alter);
    }

    /**
     * get a float FTA property.
     * @param key property name
     * @param alter alternative property in case the key was not matched
     * @return the property value
     */
    public float getFloatProperty(String key, float alter) {
	String prop = getProperty(key,null);
	if (prop==null)
	    return alter;
	try {
	    return Float.parseFloat(prop);
	}
	catch(NumberFormatException e) {
	    return alter;
	}
    }

    /**
     * get a double FTA property.
     * @param key property name
     * @param alter alternative property in case the key was not matched
     * @return the property value
     */
    public double getDoubleProperty(String key, double alter) {
	String prop = getProperty(key);
	if (prop==null)
	    return alter;
	try {
	    return Double.parseDouble(prop);
	}
	catch(NumberFormatException e) {
	    return alter;
	}
    }

    /**
     * get an FTA property. This will first search the system properties
     * and then the user property file at ~/jfta.properties.
     * @param key property name
     * @return the property value, orNULL if not found
     * @roseuid 38B3D5548908
     */
    public String getProperty(String key) {
	return getProperty(key,null);
    }

    /**
     * add local properties (won't override already existing ones!)
     * @param resource name of property file. This is typically a
     * local classpath like '/jfta/demo/foobar/blah.txt'
     * @exception IOException thrown when specified resource file couldn't
     * be loaded
     */
    public void loadLocalProperties(String resource) throws IOException {
	InputStream is = getClass().getResourceAsStream(resource);
	_userprop.load(is);
	is.close();
    }

    /**
     * get a color type property
     * @param name name of property
     * @param alt alternative color if property is not found
     * @return color data type
     */
    public Color getColorProperty(String name,Color alt) {
	String color = getProperty(name,null);
	if (color == null)
	    return alt;
	try {
	    Color col = Color.decode(color);
	    return col;
	}
	catch (NumberFormatException e) {
	    return alt;
	}
    }

    /**
     * get the hostname of the local host. This may or may not be a
     * fully qualified domainname. Better use getFQDNname
     * @see #getFQDNname(String)
     * @return the hostname
     * @roseuid 38B3D554890A
     */
    public String getHostname() {
	if (_hostname == null) {
	    try {
		// get hostname
		InetAddress addr = InetAddress.getLocalHost();
		_hostname = addr.getHostName();
	    }
	    catch (java.net.UnknownHostException e) {
		_hostname = "localhost";
	    }
	}
	return _hostname;
    }

    /**
     * send a message to the FTADaemon or dirserver at hostname
     * @param hostname hostname
     * @param addr pathname part of url
     * @return reply from FTADaemon
     * @exception IOException thrown if something went wrong
     */
    public String sendDaemonCommand(String hostname,String addr)
	throws IOException {
	URL url = getDaemonAt(hostname,addr);
	BufferedReader br = sendHTMLCommand(url);
	String line = br.readLine();
	br.close();
	return line;
    }

    /**
     * send a message to a HTTP server
     * @param url URL of server
     * @return reply from FTADaemon
     * @exception IOException thrown in case of error
     */
    public BufferedReader sendHTMLCommand(URL url)
	throws IOException {
	URLConnection uc = url.openConnection();
	return new BufferedReader
	    (new InputStreamReader(uc.getInputStream()));
    }

    /**
     * send a message to a HTTP server
     * @param url URL of server
     * @param query query string
     * @return reply from FTADaemon
     */
    public BufferedReader sendHTMLCommand(URL url,String query)
	throws IOException {
	URLConnection uc = url.openConnection();
	uc.setDoOutput(true);
	uc.setDoInput(true);
	uc.setAllowUserInteraction(false);
	DataOutputStream dos =
	    new DataOutputStream(uc.getOutputStream());
	dos.writeBytes(query);
	dos.close();
	return new BufferedReader
	    (new InputStreamReader(uc.getInputStream()));
    }

    /**
     * send a HTML query which returns a "DONE" or error
     * @param url URL of the server
     * @param query query string
     * @exception IOException thrown in case of error
     */
    public boolean sendHTMLCommandDone(URL url,String query) 
	throws IOException {
	BufferedReader br = sendHTMLCommand(url,query);
	String reply = br.readLine();
	br.close();
	return reply.startsWith("DONE");
    }

    /**
     * register a multicast address from the FTADirectory server
     * @return multicast address or null
     * @exception IOException thrown in case of error
     */
    public InetAddress addMulticast() 
	throws IOException {
	String line = sendDaemonCommand(FTADIRSERVER,
					"/cgi-bin/newmulti.ftad");
	return InetAddress.getByName(line);
    }

    public int uns(byte x) {
	return x < 0 ? (x+256) : x;
    }

    /**
     * return the next consecutively following multicast address
     * @param addr address
     * @return next addr
     * @exception IOException thrown in case of error
     */
    public InetAddress getNextMulticast(InetAddress addr) 
	throws IOException {
	byte[] a = addr.getAddress();
	String newaddr = uns(a[0]) + "." +
	    uns(a[1]) + "." +
	    uns(a[2]) + "." +
	    (uns(a[3])+1);
	return InetAddress.getByName(newaddr);
    }

    /**
     * unregister a multicast address from the FTADirectory server
     * @param addr address to remove
     * @return multicast address or null
     */
    public boolean removeMulticast(InetAddress addr) 
	throws IOException {
	String as = addr.getHostAddress();
	String line = sendDaemonCommand(FTADIRSERVER,
					"/cgi-bin/removemulti.ftad" +
					"?addr="+addr);
	return line.startsWith("DONE");
    }

    /**
     * get a unique id
     * @return an integer usable as an id
     */
    public int getUniqueId() {
	return (int)(System.currentTimeMillis() ^ (long)(Math.random()*0xfacdfacdfacdfacdl));
    }

    /**
     * get a multicast socket
     * @param addr multicast port
     * @param port multicast port
     * @param ttl time to live
     * @param buf buffersize
     * @param marsh optional marshaller
     * @return the socket
     */
    public FTABroadcastChannel getMulticastChannel(InetAddress addr,
						   int port,int ttl,
						   int buf) {
	FTABroadcastChannel chn = 
	    (FTABroadcastChannel)_multicasts.get(addr);
	if (chn == null) {
	    // create a new one
	    chn = new FTAUDPBroadcastChannel(new ThreadGroup("BC@"+addr),
					     addr,port,ttl,buf);
	    _multicasts.put(addr,chn);
	}
	return chn;
    }

    /**
     * not for private use
     */
    public void setDaemon(FTADaemon daemon) {
	if (_mydaemon != null) {
	    FTADebug.fatal("Attempt to call FTAService.setDaemon");
	} else {
	    _mydaemon = daemon;
	}
    }

    /**
     * returns the URL of the FTADaemon at hostname
     * @param host hostname
     * @param path URL path
     * @returns URL
     */
    public URL getDaemonAt(String host,String path)
	throws IOException {
	return new URL("http",host,FTADAEMONPORT,path);
    }

    /**
     * expand environment variables in a string
     * @param str input string with e.g. ${java.home} in it
     * @return converted string
     */
    public String expandEnv(String str) {
	StringBuffer sb = new StringBuffer();
	int pos = 0;
	int pos1 = str.indexOf("${",0);
	if (pos1 == -1) {
	    sb.append(str);
	} else {
	    for (;(pos1 = str.indexOf("${",pos)) >= 0; ) {
		if (pos1 > pos) {
		    sb.append(str.substring(pos,pos1));
		}
		int ebrace = str.indexOf('}',pos1+2);
		if (ebrace == -1) {
		    sb.append(str.substring(pos1));
		    break;
		} else {
		    String env = str.substring(pos1+2,ebrace);
		    pos = ebrace+1;
		    String val = getProperty(env);
		    if (val != null) {
			sb.append(val);
		    }
		}
	    }
	    sb.append(str.substring(pos));
	}
	return sb.toString();
    }

    /**
     * delay some time
     * @param time in milliseconds
     */
    public void delay(long time) {
	try {
	    Thread.sleep(time);
	} catch(InterruptedException e) {}
    }

    /**
     * convert a String to an integer, without requirement to
     * mess with NumberFormatException stuff
     * @param str String
     * @param def default value, in case of error
     * @return an int
     */
    public final int atoi(String str,int def) {
	try {
	    return Integer.parseInt(str);
	}
	catch(NumberFormatException e) {
	    return def;
	}
    }

    /**
     * return InetAddress for hostname, fail if not resolvable
     * @param hostname
     * @return address
     */
    public final InetAddress getInetAddress(String hostname) {
	try {
	    return InetAddress.getByName(hostname);
	}
	catch (java.net.UnknownHostException e) {
	    FTADebug.fatal("Cannot resolve name "+hostname);
	    return null;
	}
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
