// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTATCPCommManager.java,v 1.7 2001/06/21 14:57:10 veit Exp $

package jfta.base;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.system.*;

public class FTATCPCommManager extends FTACommManager {

    public FTATCPCommManager(FTASysSub ss,
			     String mainhost,
			     int backport,int hostnum) {
	super(ss,hostnum);
	_srvchan = new FTATCPServerChannel(_mytg);
	_srvlisteners = _srvchan.getListeners();
	_listener = new CommListener();

	// if not first host, add client for mainhost,backport
	if (hostnum != 0)
	    openClient(0,mainhost,backport);

	// add bypass (fast connection on the same host)
	_connections[hostnum] = new TCPBypass();
    }

    /**
     * add a channel to send data to a specified host
     */
    public final void openClient(int hostnum,String host,int srvport) {
	if (_connections[hostnum] == null) {	
	    FTATCPClientChannel cl = 
		new FTATCPClientChannel(host,srvport,hostnum);
	    _connections[hostnum] = cl;
	}
    }

    /*package*/ class TCPBypass extends FTAClientChannel {
	public final void sendMsg(FTAMsg msg) {
	    synchronized(_srvlisteners) {
		msg.processListener(_srvlisteners);
	    }
	}
	public final DataOutputStream getStream() {
	    return null; // not used
	}
    }

    /**
     * distribute connection information
     */
    public void distributeHostInfo() {
	// ignore the first entry (localhost)
	for (int i = 1; i< MAX_CLIENTS; i++) {
	    FTAClientChannel cl = _connections[i];
	    if (cl != null) {
		String hname = cl.getInetAddress().getHostName();
		int port = cl.getPort();
		broadcastMsg(FTAAddHostMsg.get(i,hname,port));
	    }
	}
    }
}

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
