// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAUDPBroadcastChannel.java,v 1.5 2001/06/21 14:57:10 veit Exp $

package jfta.base;

import java.net.*;
import java.util.*;
import java.io.*;

import jfta.exception.*;

public class FTAUDPBroadcastChannel extends FTABroadcastChannel {
    protected byte _myttl;
    protected MulticastSocket _mcs;
    protected DatagramPacket _snddp;
    protected int _bufsize;
    protected byte[] _rcvbuf;

    public FTAUDPBroadcastChannel(ThreadGroup tg,
				  InetAddress mcaddr,int port,
				  int ttl,int bufsize) {
	super(tg,mcaddr,port);
	_myttl = (byte)ttl;
	_bufsize = bufsize;
	_rcvbuf = new byte[_bufsize];

	byte[] sbuf = new byte[8];
	_snddp = new DatagramPacket(sbuf,sbuf.length);
	try {
	    _mcs = new MulticastSocket(port);
	    _mcs.setInterface(InetAddress.getByName(_svc.getFQDNname()));
	    _mcs.setTimeToLive(ttl);
	    _mcs.joinGroup(mcaddr);
	    _port = _mcs.getLocalPort();
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
    
	Thread thr = new UDPBRcvThread();
	thr.start();
    }

    private class UDPBRcvThread extends Thread {
	public UDPBRcvThread() {
	    super();
	}
	public void run() {
	    try {
		/* do until the socket will be closed */
		while (_mcs != null) {
		    DatagramPacket dgp = 
			new DatagramPacket(_rcvbuf,_rcvbuf.length);
		    _mcs.receive(dgp);
		    FTAMsg msg = (FTAMsg)FTAMsg.unpack(dgp.getData());
		    msg.processListener(_listeners);
		}
	    }
	    catch (Exception e) {
		/* report this: this is an internal error */
		e.printStackTrace();
	    }
	}
    }

    public final synchronized void sendMsg(FTAMsg msg) { 

	/* ignore messages, when the socket is closed */
	if (_mcs == null)
	    return;

	/* transmit the message */
	try {
	    byte [] b = msg.pack(msg);
	    _snddp.setData(b,0,b.length);
	    _snddp.setAddress(_addr);
	    _snddp.setPort(_port);
	    _mcs.send(_snddp,_myttl);
	}
	catch(Exception e) {
	    e.printStackTrace();
	}
    }

    public final void close() throws IOException {
	_mcs.close();
	_mcs = null;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
