// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAUDPServerChannel.java,v 1.4 2001/06/21 14:57:10 veit Exp $

package jfta.base;

import java.net.*;
import java.util.*;
import java.io.*;

import jfta.exception.*;

/**
 * this is an UDP server channel
 */
public class FTAUDPServerChannel extends FTAServerChannel {
    protected int _bufsize;
    protected DatagramSocket _dgs;
    protected byte[] _rcvbuf;
    
    protected FTAUDPServerChannel(ThreadGroup tg,int bufsize) {
	super(tg,null,0);
	_bufsize = bufsize;
	_rcvbuf = new byte[_bufsize];

	try {
	    _dgs = new DatagramSocket();
	    _addr = _dgs.getLocalAddress();
	    _port = _dgs.getLocalPort();
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
	
	Thread thr = new UDPRcvThread(_systg);
	thr.start();
    }
    
    private class UDPRcvThread extends Thread {
	public UDPRcvThread(ThreadGroup tg) {
	    super(tg,"UDPRcv");
	}

	public void run() {
	    try {
		/* repeat until socket is closed */
		while (_dgs != null) {
		    DatagramPacket dgp =
			new DatagramPacket(_rcvbuf,_rcvbuf.length);
		    _dgs.receive(dgp);
		    FTAMsg msg = (FTAMsg)FTAMsg.unpack(dgp.getData());
		    msg.processListener(_listeners);
		}
	    }
	    catch (Exception e) {
		_dgs.close();
		_dgs = null;
	    }
	}
    }

    public final void close() throws IOException {
	_dgs.close();
	_dgs = null;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
