// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTADDirServlet.java,v 1.4 2001/06/21 14:57:10 veit Exp $

package jfta.bootstrap;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.base.*;
import jfta.exception.*;
import jfta.http.*;

public class FTADDirServlet extends FTAServlet {

    private static FTAService _svc = FTAService.Instance();
    private int _updatelatency;
    private FTADaemon _daemon;

    /**
     * mandatory constructor
     */
    public FTADDirServlet() {
	super();
    }

    /**
     * init method
     */
    public void init(FTAHttpServer server,Vector argv)
	throws FTAArgumentException {

	// get property
	_updatelatency =
	    _svc.getIntegerProperty("fta_daemon_update_latency",5);

	_daemon = (FTADaemon)server.getContainer();
    }
    
    /**
     * reply method.
     */
    public FTAServletReply reply(FTAServletRequest req) {
	String head = req.getHead();
	Hashtable header = req.getHeader();
	Hashtable args = req.getArgs();
	String orghost = (String)header.get("X-Originating-Host");
	long now = System.currentTimeMillis();
	Date dnow = new Date(now);
	boolean isdirserver = _daemon.isDirServer();

	try {
	    if (isdirserver) {
		if (head.equals("/newmulti.ftad"))
		    return newMulti(orghost);
		else if (head.equals("/removemulti.ftad"))
		    return removeMulti(args);
		else if (head.equals("/removemulti.ftad"))
		    return removeMulti(args);
		else if (head.equals("/reset.ftad")) {
		    reset();
		    head = "/dir.html";
		}
		if (head.equals("/dir.html"))
		    return doDir(dnow);
	    }
	    if (head.equals("/finddir.ftad")) {
		findDirServer(now);
		head = "/dir.html";
	    }

	    if (head.equals("/dir.html")) {
		return doNonDir();
	    } else
		return FTAErrorReply.error404();
	}
	catch (Exception e) {
	    // can't be handled: internal error
	    return FTAErrorReply.error500(e.toString());
	}
    }
    
    private FTAServletReply doDir(Date dnow) {
	FTAHtmlFile f = new FTAHtmlFile("FTADirServer at " + 
			    _svc.getFQDNname());
	String dirurl = _daemon.getDirServer();
	f.h2("Properties");
	f.beginUL();
	f.item("Service URL: "); f.b("/cgi-bin/newmulti.ftad");
	f.item("Current time: "); f.b(dnow.toString());
	f.item("Address pool: "); f.b(_daemon.getAddrPool());
	f.item("Time To Live: "); 
	f.b(_svc.getIntegerProperty("fta_multicast_ttl",3)+"");
	f.item("Currently active: ");
	f.b(_daemon.getAddrPoolSize() + " addresses");
	f.endUL();
	f.beginForm(dirurl + "/cgi-bin/reset.ftad");
	f.submitButton("Reset Address pool");
	f.endForm();
	return _daemon.stdFinish(f);
    }

    private FTAServletReply doNonDir() {
	FTAHtmlFile f = new FTAHtmlFile("Locate FTADirServer");
	String dirurl = _daemon.getDirServer();
	f.beginForm("/cgi-bin/finddir.ftad");
	f.submitButton("Find DirServer");
	f.endForm();
	f.p("("+_updatelatency+" seconds latency)");
	f.beginUL();
	if (dirurl != null) {
	    f.item("The DirServer is at ");
	    f.anchor(dirurl+"/dir.html",dirurl);
	} else {
	    f.item("No DirServer known yet");
	}
	f.endUL();
	return _daemon.stdFinish(f);
    }

    private void findDirServer(long now)
	throws InterruptedException {
	_daemon.sendMsg(FTADirReqMsg.get(now));
	_svc.delay(_updatelatency * 1000);
    }

    private FTAServletReply newMulti(String orghost) {
	String newaddr = _daemon.getNewAddr(orghost);
	if (newaddr != null) {
	    FTAFile f = new FTAFile(10);
	    f.add(newaddr);
	    return f;
	}
	return FTAErrorReply.error404();
    }

    private FTAServletReply removeMulti(Hashtable args) 
	throws IOException {
	String addr = (String)args.get("addr");
	if (_daemon.removeAddr(addr))
	    return _daemon.replyDone();
	return FTAErrorReply.error404();
    }

    private void reset() throws IOException {
	_daemon.resetAddrs();
    }

    private FTAServletReply listDaemons() {
	return _daemon.listall(_daemon.getDaemons());
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
