// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: Req3Tag.java,v 1.2 2001/06/21 14:57:13 veit Exp $


package jfta.demo.cs3;

import jfta.exception.*;
import jfta.tags.*;
import jfta.http.*;

/**
 * This tag stores a request for the sample client/server system
 * It is an example of a tag to be used only by the new constructor based
 * method, since init() is not implemented, it will cause a baseclass
 * exception when you try to use it in the old way. The better alternative is
 * shown in Reply3Tag, where both methods are implemented.
 * @author Holger Veit
 * @version $Id: Req3Tag.java,v 1.2 2001/06/21 14:57:13 veit Exp $
 */
public class Req3Tag extends FTATag {
    
    /**
     * attributes
     */
    private int clid;
    private int reqid;
    private int oper;
    private int arg1;
    private int arg2;
    
    /**
     * access methods
     */
    public int getClientId() {
	return clid;
    }
    public int getRequestId() {
	return reqid;
    }
    public int getArg1() {
	return arg1;
    }
    public int getArg2() {
	return arg2;
    }
    public int getOper() {
	return oper;
    }
    
    /**
     * approx # of tags on tagboard
     * @return the approx. number of tags to be expected during a flip cycle
     */
    public int getCapacityHint() {
	return 50;
    }
    
    /**
     * empty constructor
     */
    public Req3Tag() {
	super();
    }
    
    /**
     * custom constructor
     */
    public Req3Tag(int cid,int rid,int a1,int a2,int op) {
	super();
	clid = cid;
	reqid = rid;
	arg1 = a1;
	arg2 = a2;
	oper = op;
    }

    /**
     * provide HTML Information about this object and its 
     * contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHtmlFile f = new FTAHtmlFile("Tag info for ReqTag");
	f.beginUL();
	f.propitem("Hashcode",hashCode()+"");
	f.propitem("Client Id",clid+"");
	f.propitem("Request Id",clid+"");
	f.propitem("Argument 1",arg1+"");
	f.propitem("Operation",oper+"");
	f.propitem("Argument 2",arg2+"");
	f.finish();
	return f;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
