// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This is a template for an own actor type
// insert stuff at all locations that have [ xxxxx ]
// by default, this file cannot be compiled.
// $Id: Input.java,v 1.2 2001/06/21 14:57:16 veit Exp $

package jfta.demo.input;

import jfta.base.*;
import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.http.*;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * This actor communicates with an external InputApplet
 * @author Holger Veit
 * @version $Id: Input.java,v 1.2 2001/06/21 14:57:16 veit Exp $
 */
public class Input extends FTAActor implements FTAActorI {
    
    /**
     * socket for Applet communication
     */
    private ServerSocket _sock;
    
    private String _server;
    private int _port;

    private int _conncnt;

    /**
     * The handleSession method does all the communication of a new
     * connection to the InputApplet. In this case, it just receives
     * data, echoes it back to the Applet, and copies it to the
     * userMessage log.
     */
    private void handleSession(FTAActor ac,
			       BufferedReader in,PrintStream out)
	throws Exception {

	_conncnt++;
	int mycnt = _conncnt;
	
	// perform "open" protocol
	// incoming applet is supposed to send "open", and
	// actor replies with "ok". After that, both partners
	// are locked.
	String line = in.readLine().toLowerCase();
	if (!line.startsWith("open"))
	    return;
	out.println("ok");

	ac.userMessage(LOG_STD,"New Connection "+mycnt);
	for (;;) {
	    line = in.readLine().toLowerCase();
	    ac.userMessage(LOG_STD,"Conn "+mycnt+": Input="+line);
	    out.println("Echo: "+line);
	    if (line.startsWith("exit")) {
		ac.userMessage(LOG_STD,"Conn "+mycnt+": Closed Session");
		return;
	    }
	}
    }

    /**
     * the Handler thread manages a single session.
     * It listens for input data and if unblocked, passes it
     * to the method handleSession(). This method will
     * return after the communication is closed.
     */
    private class HandlerThread extends Thread {
	private Socket _conn;
	private FTAActor _ac;
	public HandlerThread(Socket conn, FTAActor ac) {
	    super();
	    _conn = conn;
	    _ac = ac;
	}
				    
	public void run() {
	    BufferedReader in;
	    PrintStream out;
	    try {
		in = new BufferedReader(new
		    InputStreamReader(_conn.getInputStream()));
		out = new PrintStream(_conn.getOutputStream());
		handleSession(_ac,in,out);
		_conn.close();
	    } catch (Exception e) {}
	}
    }

    /**
     * The server thread maintains the public server socket.
     * it blocks until a new connection is accepted and
     * then spawns off a new handler thread.
     */
    private class ServerThread extends Thread {
	private ServerSocket _sock;
	private FTAActor _ac;
	public ServerThread(ServerSocket s,FTAActor ac) {
	    super();
	    _sock = s;
	    _ac = ac;
	}
	
	/**
	 * the run method of the server thread continuously listens
	 * for new connections, and if connected, will spawn a new
	 * handler thread.
	 */
	public void run() {
	    try {
		while (_sock != null) {
		    Socket conn = _sock.accept();
		    HandlerThread thr2 = new HandlerThread(conn,_ac);
		    thr2.start();
		}
	    } catch (Exception e) {}
	}
    }

    /**
     * mandatory eval method. This does nothing here.
     * The writer of a real application will probably have to
     * add code for communication with the Handler thread to get or
     * send data.
     *
     * @return FTAActor.TERMINATE if the actor terminates after this
     *   routine has been called, FTAActor.KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {
	return KEEPALIVE;
    }

    /**
     * mandatory init method
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incoreect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {

	try {
	    // create a server socket to listen on
	    _sock = new ServerSocket(0,10);
	    _server = _svc.getFQDNname();
	    _port = _sock.getLocalPort();
	}
	catch(IOException e) {
	    throw new FTAArgumentException(e.getMessage());
	}

	// create a server thread to process an incoming connection
	ServerThread thr = new ServerThread(_sock,this);
	thr.start();
    }
    
    /**
     * mandatory empty constructor
     */
    public Input() {
	super();
    }

    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHtmlFile f = new FTAHtmlFile("Input Actor");
	f.text("You must have enabled 'Java' in your browser!");
	f.hr();
	Hashtable params = new Hashtable();
	params.put("server",_server);
	params.put("port",new Integer(_port));

	// this calculates the applet path prefix
	String prefix = "http://" + 
	    _server + ":" + FTAService.FTADAEMONPORT + 
	    "/applets/";

	// insert the applet
	f.applet(prefix+"jfta/demo/input",
		 "InputApplet.class",
		 640,480,params);
	f.hr();
	return f.finish();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 *
 */
