// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: InputApplet.java,v 1.2 2001/06/21 14:57:16 veit Exp $

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;

/**
 * This class is the client-side interface of the FTA input demo
 *
 * Insert it into a webform with
 *   <APPLET CODE="http://server:port/pathto/InputApplet.class"
 *      WIDTH=640 HEIGHT=480>
 *   <PARAM NAME="server" VALUE="server.na.me">
 *   <PARAM NAME="port" VALUE=12345>
 *   </APPLET>
 *
 * Note that you have to obtain the port to connect to through some other
 * method, like looking on the appropriate web page. By default,
 * the page calling this applet is generated by the Input actor
 */

public class InputApplet extends Applet {

    private Socket sock;
    private BufferedReader in;
    private PrintStream out;
    private TextArea ta;
    private TextField tf;

    private void append(String text) {
	if (ta != null) {
	    ta.append(text+"\r\n");
	}
    }

    private class ReaderThread extends Thread {
	private BufferedReader in;
	public ReaderThread(BufferedReader in) {
	    super();
	    this.in = in;
	}

	public void run() {
	    try {
		String resp;
		while ((resp=in.readLine()) != null) {
		    append(resp);
		}
	    } catch(IOException e) {
		ta.append("\nConnection closed by foreign host\n");
	    }
	}
    }

    private boolean connect() {
	// retrieve the connection parameters
	String server = getParameter("server");
	String sport = getParameter("port");
	if (server==null || sport==null) {
	    return false;
	}
	int port;
	try {
	    // create a connection socket
	    port = Integer.parseInt(sport);
	    sock = new Socket(server,port);

	    // get connection streams
	    in = new BufferedReader(new
		InputStreamReader(sock.getInputStream()));
	    out = new PrintStream(sock.getOutputStream());

	    // send connect request to Actor. The actor is
	    // supposed to get the 'open' and reply with 'ok'
	    out.println("open");
	    String resp = in.readLine();
	    if (resp.equals("ok")) {
		ReaderThread thr = new ReaderThread(in);
		thr.start();
	    }
	}
	catch (Exception e) {
	    return false;
	}
	return true;
    }

    private void disconnect() {
	try {
	    out.println("close");
	    out.flush();
	    sock.close();
	} catch (Exception e) {}
	sock = null;
    }

    public void init() {

	boolean cn = connect();

	this.setLayout(new BorderLayout());
	ta = new TextArea("",15,80,TextArea.SCROLLBARS_BOTH);
	ta.setBackground(Color.white);
	ta.setEditable(false);
	this.add("Center",ta);
	Panel p = new Panel();
	p.setLayout(new BorderLayout());
	Label l = new Label("Enter line:");
	p.add("West",l);
	tf = new TextField(60);
	tf.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String text = tf.getText();
		    if (text.equals("clear")) {
			ta.setText("");
		    } else if (text.equals("exit")) {
			out.println(text);
			destroy();
		    } else {
			if (out != null) out.println(text);
		    }
		    tf.setText("");		}
	    });
	p.add("East",tf);
	this.add("South",p);
    }

    public void start() {
	tf.setEnabled(true);
    }

    public void stop() {
	tf.setEnabled(false);
    }


    public void destroy() {
	disconnect();
	remove(tf);
	remove(ta);
    }

    public void paint(Graphics g) {
	super.paint(g);
    }

    public String getAppletInfo() {
	return "Input";
    }

    private String[][] paraminfo = {
	{ "server", "String", "hostname.do.main of the FTASystem" },
	{ "port", "int", "TCP server port of the corresponding actor" }
    };

    public String[][] getParameterInfo() {
	return paraminfo;
    }
}
