// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This is a template for an own actor type
// insert stuff at all locations that have [ xxxxx ]
// by default, this file cannot be compiled.
// $Id: MultiV.java,v 1.3 2001/06/21 14:57:16 veit Exp $

package jfta.demo.multi;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.base.*;

/**
 * This actor calculates the average fitness of a population
 * @author Holger Veit
 * @version $Id: MultiV.java,v 1.3 2001/06/21 14:57:16 veit Exp $
 */
public class MultiV extends FTAActor implements FTAActorI {
    
    /**
     */
    private long lastSeen;
    
    /**
     * mandatory eval() method
     * @return TERMINATE if the actor terminates after this
     *   routine has been called, KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException {

	long now = getTagboardTime("tb",RELATIVE);
	if (now <= lastSeen)
	    return KEEPALIVE;
	lastSeen = now;

	double fitness = 0.0;
	int ivcount = 0;
	FTATagIterator it = getTagIterator("tb","jfta.demo.multi.Individual");
	while (it.hasNext()) {
	    Individual iv = (Individual)it.next();
	    ivcount++;
	    fitness += iv.getFitness();
	}

	if (ivcount > 0) {
	    fitness /= ivcount;
	    userMessage(LOG_STD,"Time="+now+
			" fitness="+fitness+" pop="+ivcount);
	}
	return KEEPALIVE;
    }
    
    /**
     * mandatory init() method
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incorrect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {
	throw new FTAArgumentException("Use explicit constructor");
    }
    
    /**
     */
    public MultiV() {
	super();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
