// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This is a template for an own actor type
// insert stuff at all locations that have [ xxxxx ]
// by default, this file cannot be compiled.
// $Id: Output.java,v 1.2 2001/06/21 14:57:17 veit Exp $

package jfta.demo.output;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.http.*;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * This actor will open a socket connection to an external server
 * and will send out periodic messages to this server.
 * @author Holger Veit
 * @version $Id: Output.java,v 1.2 2001/06/21 14:57:17 veit Exp $
 */
public class Output extends FTAActor implements FTAActorI {
    
    private Socket sock;
    private BufferedReader in;
    private PrintStream out;
    private long lasttime;
    private long calls;
    private boolean connected;
    
    private LinkedList inputqueue;
    
    private void enqueue(Object o) {
	synchronized(inputqueue) {
	    inputqueue.addFirst(o);
	}
    }
    
    private Object dequeue() {
	synchronized(inputqueue) {
	    return inputqueue.removeFirst();
	}
    }
    
    private boolean dataAvailable() {
	return inputqueue.size() > 0;
    }
    
    private class ReaderThread extends Thread {
	private BufferedReader in;
	public ReaderThread(BufferedReader in) {
	    super();
	    this.in = in;
	}
	
	public void run() {
	    try {
		String resp;
		while ((resp=in.readLine()) != null) {
		    enqueue(resp);
		}
	    } catch(IOException e) {
		userMessage(LOG_ERROR,"Connection closed by foreign host");
		connected = false;
	    }
	}
    }

    private boolean connect() {

	// retrieve the connection parameters
	String server = _svc.getFQDNname();
	int port = OutputServer.PUBLICSOCK;

	try {
	    // create a connection socket
	    sock = new Socket(server,port);

	    // get connection streams
	    in = new BufferedReader(new
		InputStreamReader(sock.getInputStream()));
	    out = new PrintStream(sock.getOutputStream());

	    // send connect request to server. The server is
	    // supposed to get the 'open' and reply with 'ok'
	    out.println("open");
	    String resp = in.readLine();
	    if (resp.startsWith("ok")) {
		// start the optional reader thread here
		ReaderThread thr = new ReaderThread(in);
		thr.start();
		return true;
	    }
	}
	catch (Exception e) {}
	return false;
    }

    private void disconnect() {
	try {
	    out.println("close");
	    out.flush();
	    sock.close();
	} catch (Exception e) {}
	sock = null;
    }
    
    /**
     * mandatory eval() method
     * @return TERMINATE if the actor terminates after this
     * routine has been called, KEEPALIVE to have the actor
     * be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal
     * error occurred
     */
    public int eval() throws FTAEvalActorException {
	// count the number of activations of this actor
	calls++;
	
	// check for new data
	while (dataAvailable()) {
	    String resp = (String)dequeue();
	    userMessage(LOG_SUCCESS,resp);
	}

	// each second, try to emit a single line
	long time = System.currentTimeMillis();
	if ((time-lasttime) > 1000) {
	    lasttime = time;

	    // send a message to server
	    if (connected) {
		out.println("Time is "+time+" (actor called "+calls+" times)");
		calls = 0;
	    } else {
		// try to connect
		if (connect()) 
		    connected = true;
		else
		    userMessage(LOG_STD,
				"Time is "+time+" (not connected yet)");
	    }
	}
	return KEEPALIVE;
    }
    
    /**
     * init() method
     * @param args list of arguments passed by <code>addActor</code>
     * to the newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code
     * if some argument is incorrect for this type of actor
     */
    public void init(String[] args) throws FTAArgumentException {
	// does nothing
    }
    
    /**
     * post initialize the actor
     * This method is called when the actor is received at the
     * target ensembleSub before it is stored. This allows to
     * do late initializations. The default method does nothing.
     */
    public void postInit() {
	lasttime = System.currentTimeMillis();
	inputqueue = new LinkedList();
    }

    /**
     */
    public Output() {
	super();
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
