// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.ping;

import jfta.actors.*;
import jfta.exception.*;
import jfta.tags.*;

public class PingActor extends FTAActor implements FTAActorI { 
    private String[] msgs; 
    private int count;
    private TDContainer container;
    private long absolutetime,relativetime,acctime,globaltime,globaltagcnt,
	globalnonflip,tagcount,acctrip,globaltrip;
    private static final int HOSTS = 7;
    
  public void init(String args[])throws FTAArgumentException{
      // tag content
      msgs = new String[2];
      // actor calls
      count = 0;  
      // accumulated absolute times
      acctime = 0;
      // tags received
      tagcount = 0;
      // accumulated trip time
      acctrip = 0;

      // relative ensemble time
      relativetime = 0;
      // absolute ensemble time
      absolutetime = 0;

      container = new TDContainer();
      container.add("tbd0","write",this);
      for(int i = 1; i < HOSTS; i++)
	  container.add("tbd" + i,"read",this);

      // overall time
      globaltime = 0;
      // overall tag count
      globaltagcnt = 0;
      // global trip time
      globaltrip = 0;
  }

    /**
       prints messages 
     */
  private void printActor(String msg){
    System.out.println(getName() + "["+count+"]: " + msg);
  }
    
    /**
       increase actor calls count
     */
  private void incTime(){
    ++count;
  }
  
    /**
       sends a ping tag
     */
  private void sendPing()
    throws FTATagException,FTAArgumentException,
	   FTAUnknownObjectException,TagboardDataUnknownException{
      container.putTag("jfta.demo.ping.PingTag",msgs);
  }

    /**
       receives pong tag and extracts information
     */
    private void recvPong(String tbname)
	throws NumberFormatException,FTAArgumentException,
	       FTAUnknownObjectException,TagboardDataUseException,
	       TagboardDataUnknownException{
	long triptime, tripcycles;
	PongTag tag = null; 
	FTATagIterator it = 
	    container.getTagIterator(tbname, new FTATagPattern("jfta.demo.ping.PongTag"));
	
	try{	
	    while(it.hasNext())
		{
		    tag = (PongTag) it.next(); 
		    
		    triptime   = calcTripTime(tag,ABSOLUTE);
		    tripcycles = calcTripTime(tag,RELATIVE);
		    
		    acctrip += tripcycles;
		    acctime += triptime;
		    tagcount++;
		}
	}
	catch(FTATagboardFlipException e){
	    printActor("tagboard has flipped."); 
	}
    }
    
    
    /**
       calculate trip times
     */
    private long calcTripTime(PongTag tag, int format){
	if(format == RELATIVE)
	    return relativetime -  
		Long.parseLong(extractTime(tag,RELATIVE));
	else
	    return absolutetime -
		Long.parseLong(extractTime(tag,ABSOLUTE));
    }

    /**
       extract times
     */
  private String extractTime(PongTag tag, int format){
      String content = tag.getContent();
      try{
	  int idx = content.indexOf(":");
	  
	  if(format == ABSOLUTE)
	      return content.substring(0,idx);
	  else
	      return content.substring(idx+1,content.length());
      }
      catch(Exception e){System.exit(1);}
      return null;
  } 

    /**
       get ensemble time (relative & absolute)
     */
  private void ensembleTime()
    throws FTAArgumentException,FTAUnknownObjectException{
      absolutetime = 0;
      relativetime = 0;
      while((absolutetime == 0) && (relativetime == 0) ){
	  absolutetime = getEnsembleTime("end0",ABSOLUTE);
	  relativetime = getEnsembleTime("end0",RELATIVE);
      }
  }
  
    private double longToDouble(long number){
	return (new Long(number)).doubleValue();
    }

    private String getStringVal(double number){	
	number = longToDouble(Math.round(number * 100)) / 100.0; 
	return (new Double(number)).toString();
    }

    /**
       send ping and receive pong
     */
    public int eval() throws FTAEvalActorException{
	try {
	    //  increase time
	    incTime();
	    
	    // get/set ensemble time
	    ensembleTime();
	    
	    // copy to message  
	    msgs[0] = Long.toString(absolutetime);
	    msgs[1] = Long.toString(relativetime);
 
	    // tagboard flipped => send ping
	    sendPing();
	    for(int i = 1;i < HOSTS;i++){
		if(container.hasFlipped("tbd" + i))
		    recvPong("tbd" + i);
	    } 
	
	    // flipOrder("tbd0");
	    
	    if((count % 100)==0)
		{
		    double avgtime,avgtrip = 0.0;
		    if(tagcount > 0){
			avgtime = longToDouble(acctime)
			    / longToDouble(tagcount);
			avgtrip = longToDouble(acctrip) 
			    / longToDouble(tagcount);
		    }
		    else
			avgtime = longToDouble(acctime);

		    printActor("avg ping(msec (end0)): " +
			       getStringVal(avgtime) + 
			       " #tags " + tagcount + 
			       // + td0.getName()+
			       // "(non flips): " + td0.getNonFlips() + 
			       //" " + td1.getName() + "(non flips): " +
			       // td1.getNonFlips() + 
			       " avg trip(#ticks (end0)): " + 
			       getStringVal(avgtrip));
		    
		    globaltime    += acctime;
		    globaltagcnt  += tagcount;
		    globaltrip    += acctrip;

		    acctime  = 0;
		    acctrip  = 0;
		    tagcount = 0;

		}
	    if(count < 1000)
		return KEEPALIVE;
	    else {
		double globalping = Double.longBitsToDouble(globaltime) 
		    / Double.longBitsToDouble(globaltagcnt);
		double globaltripcycles = Double.longBitsToDouble(globaltrip)
		    / Double.longBitsToDouble(globaltagcnt);

		// print result
		printActor("overall performance: #cycles: "+count+
			   " - ping (msec): "+ getStringVal(globalping) + 
			   " - #tags: "+ globaltagcnt + 
			   // td0.getName() + "(non flips): " + 
			   //td0.getGlobalNF() +" " + td1.getName() +
			   //"(non flips): " + td1.getGlobalNF() +
			   " - trip (cycles): " + 
			   getStringVal(globaltripcycles));
		return TERMINATE;
	    }
	}
	catch(Exception e) {
	    System.out.println("Exception raised: " + e);
	    e.printStackTrace();
	    return TERMINATE;
	}
    }
    public PingActor(){super();}
}
