// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: DisplayActor.java,v 1.2 2001/06/21 14:57:18 veit Exp $

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

import java.util.*;

/**
  * This actor manages a separate Displayer. It reads the
  * Environment Tagboard and submits changes to a visualizer frame.
  * @author Holger Veit
  * @version $Id: DisplayActor.java,v 1.2 2001/06/21 14:57:18 veit Exp $
  */
public class DisplayActor extends FTAActor implements FTAActorI,Constants {

  private static String WhoAmI = "DisplayActor";

  /**
    * work variables
    */
  private FTATBCallback cb;
  private String title;
  private DisplayI viewer;

  /**
    * create the viewer
    */
  private boolean setup_viewer(String viewclass,String title) {
    try {
      // create an instance
      Class cls = Class.forName(viewclass);
      Object obj = cls.newInstance();
      if (obj instanceof DisplayI)
	viewer = (DisplayI)obj;
      else
	return false;
      
      // initialize it
      viewer.init(this,title,640,480);
      return true;
    }
    catch (Exception e) {
      e.printStackTrace();
      return false;
    }
  }

  /**
    * mandatory eval routine
    * @return TERMINATE if the actor terminates after this
    *   routine has been called, KEEPALIVE to have the actor
    *   be rescheduled in the next tick cycle.
    * @exception FTAEvalActorException thrown in case a fatal error occurred
    */
  public int eval() throws FTAEvalActorException {

    if (viewer == null || viewer.hasTerminated()) {
      userMessage(WhoAmI,"EXITING...");
      return TERMINATE;
    }
    return cb.eval();
  }

  private class MapCallback extends FTATBCallback {
    public MapCallback(FTAActor ac) {
      super(ac,enviTB);
    }
    public int eval2() throws FTAEvalActorException {
      DynObjectI obj;
      ModObjectI mobj;

      /* 1. count args and detect a SWALL tag: clear everything */
      FTATagIterator iter;
      boolean clear = false;
      for  (iter = getTagIterator(enviTB,mappattern); iter.hasNext(); ) {
	obj = (DynObjectI)iter.next();
	if (!clear && obj instanceof MapObjTag &&
	    ((MapObjTag)obj).getObjType() == MapObjTag.MAPSWALL)
	  clear = true;
      }

      /* clear items if a SWALL tag has been found */
      if (clear) viewer.removeAllObjects();

      /* 2. add objects */
      for  (iter = getTagIterator(enviTB,mappattern); iter.hasNext(); ) {
	obj = (DynObjectI)iter.next();
	viewer.addObject(obj);
      }

      /* 3. modify objects */
      for  (iter = getTagIterator(enviTB,modpattern); iter.hasNext(); ) {
	mobj = (ModObjectI)iter.next();
	viewer.modifyObject(mobj);
      }

      /* update the display */
      viewer.update();

      return KEEPALIVE;
    }
  }
  
  /**
    * mandatory init method
    * @param args list of arguments passed by <code>addActor</code> to the
    *   newly created actor. 
    * @exception FTAArgumentException to be thrown by the user code if some
    *   argument is incorrect for this type of actor
    * @return returns true, if the initialization was okay, false if there
    *   has been a problem not requiring an <code>FTAArgumentException</code>
    */
  public void init(String[] args) throws FTAArgumentException {
    String viewclass = DEFAULT_DISPLAY;

    // one or two args
    // arg1: title
    // optional arg2: DisplayerClass

    if (args.length < 1)
      throw new FTAArgumentException(WhoAmI+" requires argument");
    title = args[0];
    if (args.length == 2)
      viewclass = args[1];

    userMessage("DisplayActor"," START: "+title);

    if (!setup_viewer(viewclass,title))
      throw new FTAArgumentException(WhoAmI+" can't setup viewer "+
				     viewclass);
  }
  
  /**
    * mandatory empty constructor
    */
  public DisplayActor() {
    super();

    cb = new MapCallback(this);
  }
}
