// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This standalone utility is a simple Input window that will
// connect to a SimServer on some host
// and will submit commands to it
// $Id: KbdController.java,v 1.2 2001/06/21 14:57:18 veit Exp $

package jfta.demo.simrob;

import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import jfta.base.*;
import jfta.base.gui.*;
import jfta.exception.*;

public class KbdController extends Frame implements ActionListener {
  private SimServerI myserver;
  private String _hostname,_instname;

  private void connectme() throws FTAException {
    if (myserver==null) {
      myserver = SimServerRemote.getRemote(_hostname,_instname);
    }
  }

  public static void main(String[] args) {

    KbdController myself = new KbdController();
    
    if (args.length < 1)
      myself.usage();

    FTAService svc = FTAService.Instance();
    String hostname = svc.getFQDNname(args[0]);
    String instname = null;
    if (args.length >= 2)
      instname = args[1];

    myself.init(hostname,instname);
    myself.setVisible(true);
  }

  public void usage() {
    System.out.println("usage: KbdController hostname [instname]");
    System.exit(1);
  }

  private void help(String helpfile) {
    FTAGUIHelpWindow hw = new FTAGUIHelpWindow(this,"Command Help",helpfile);
    hw.show();
  }

  private void beginwait() {
    setCursor(Cursor.WAIT_CURSOR);
    _cmdline.setEditable(false);
  }

  private void endwait() {
    setCursor(Cursor.DEFAULT_CURSOR);
    _cmdline.setEditable(true);
  }

  public void actionPerformed(ActionEvent e) {
    Object obj = e.getSource();
    if (obj instanceof TextField) {
      TextField tf = (TextField)obj;
      String cmd = tf.getText();
      _transcript.append(cmd+"\n");
      sendcmd(cmd);
      if (cmd.equals("quit"))
	exit();
    } else if (obj instanceof MenuItem) {
      FTAGUIMenuItem m = (FTAGUIMenuItem)obj;
      if (m.isequal("Clear Transcript")) {
	clear();
      } else if (m.isequal("Save Transcript")) {
        save();
      } else if (m.isequal("Print Setup")) {
        printsetup();
      } else if (m.isequal("Print")) {
        print();
      } else if (m.isequal("Exit")) {
        exit();
      } else if (m.isequal("Command Help")) {
        help("/jfta/demo/simrob/cmdhelp.txt");
      }
    }
  }

  private void exit() {
    System.exit(0);
  }

  private void printsetup() {
    prt.printSetup();
  }

  private void print() {
    prt.print(_transcript);
  }

  private void sendcmd(String cmd) {
    _cmdline.setText(null);
    try {
      beginwait();
      connectme();
      myserver.sendCmd(cmd);
      endwait();
    }
    catch(Exception ex) {
      endwait();
      Vector buttons = new Vector();
      buttons.add("1/Fail&Exit");
      buttons.add("2/Ignore");
      FTAGUIMsgBox eb = new FTAGUIMsgBox(this,
					 "Connect Error",
					 "Cannot send command to SimServer",
					 "/jfta/base/gui/error.gif",buttons);
					 
      eb.setVisible(true);
      //JOptionPane.showConfirmDialog(this,
      //				    "Cannot send command to SimServer",
      //				    "Connect Error",
      //				    JOptionPane.OK_OPTION,
      //				    JOptionPane.ERROR_MESSAGE);
      if (eb.getReply()==1)
        System.exit(1);
    }
  }

  private void clear() {
    _transcript.setText("");
  }

  /**
    * save text
    */
  private void save() {
    FileDialog fd = new FileDialog(this,"Save Transcript",FileDialog.SAVE);
    fd.show();
    String fn = fd.getFile();
    if (fn != null) {
      File fi = new File(fd.getDirectory(),fn);
      save(fi,_transcript.getText());
    }
  }
    
  /**
    * save text into file
    */
  private void save(File fi,String text) {
    try {
      beginwait();
      PrintWriter pw = new PrintWriter(new FileOutputStream(fi));
      pw.write(text);
      pw.close();
      endwait();
    }
    catch(Exception e) {
      endwait();
      FTAGUIErrorBox eb = new FTAGUIErrorBox(this,
					     "Save Error","Cannot save File");
      eb.show();
    }
  }  


  private TextField _cmdline;
  private FTAGUITextArea _transcript;
  private transient FTAGUIPrintManager prt;

  public KbdController() {
    super("KbdController");
    prt = new FTAGUIPrintManager(this);
  }

  private void add_menus() {
    MenuBar mb = new MenuBar();

    Menu fmenu = new Menu("File");
    new FTAGUIMenuItem(fmenu,"Save Transcript",KeyEvent.VK_S,this);
    fmenu.addSeparator();
    FTAGUIMenuItem it =
      new FTAGUIMenuItem(fmenu,"Print Setup",KeyEvent.VK_R,this);
    it.setEnabled(false);
    new FTAGUIMenuItem(fmenu,"Print",KeyEvent.VK_P,this);
    fmenu.addSeparator();
    new FTAGUIMenuItem(fmenu,"Exit",KeyEvent.VK_X,this);
    mb.add(fmenu);

    Menu emenu = new Menu("Edit");
    new FTAGUIMenuItem(emenu,"Clear Transcript",KeyEvent.VK_C,this);
    mb.add(emenu);

    setMenuBar(mb);
    
    Menu hmenu = new Menu("Help");
    new FTAGUIMenuItem(hmenu,"Command Help",KeyEvent.VK_H,this);
    mb.setHelpMenu(hmenu);
  }

  private void init(String hname,String iname) {
    _hostname = hname;
    _instname = iname;

    add_menus();

    setLayout(new BorderLayout());

    Panel view_p = new Panel();
    view_p.setLayout(new FlowLayout());

      Label hlabel = new Label("host: ");
      TextField host = new TextField(_hostname,20);
      host.setBackground(Color.white);
      host.setEditable(false);
    view_p.add(hlabel);
    view_p.add(host);

      Label ilabel = new Label("instance: ");
      TextField inst = new TextField(_instname==null?"DEFAULT":_instname,20);
      inst.setBackground(Color.white);
      inst.setEditable(false);
    view_p.add(ilabel);
    view_p.add(inst);

    Panel text_p = new Panel();
    text_p.setLayout(new BorderLayout());

      Label tlabel = new Label("Enter your command:");
      _cmdline = new TextField(40);
      _cmdline.setBackground(Color.white);
      _cmdline.addActionListener(this);

    text_p.add(tlabel,"North");
    text_p.add(_cmdline,"Center");

    Panel trans_p = new Panel();
    trans_p.setLayout(new BorderLayout());
    _transcript = new FTAGUITextArea("",20,40,TextArea.SCROLLBARS_BOTH);
    _transcript.setBackground(Color.white);
    _transcript.setEditable(false);

    trans_p.add(new Label("Transcript:"),"North");
    trans_p.add(_transcript,"Center");

    add(view_p,"North");
    add(text_p,"Center");
    add(trans_p,"South");

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        exit();
      }
    });

    pack();
    setLocation(100,100);
    setVisible(false);
  }
}
