// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// This class will parse a line-structured file

package jfta.demo.simrob;

import java.io.*;
import java.util.*;

/**
  * This class parses a line-structured file 
  * where each line (except empty and comment lines is started with
  * a keyword and a number of numeric or non-numeric arguments
  * Requires:
  *   a cmd list where each commands implements the LineParse interface
  * @author Holger Veit
  * @see LineParse
  */
public class LineParser {

  public static boolean getBool(StreamTokenizer st)
    throws LineParserException {
      String[] tf = { "false", "true" };
      int i = getKeyword(st,tf,false);
      return i==1;
  }

  public static int getInt(StreamTokenizer st)
    throws LineParserException {
      try {
	if (st.nextToken() != StreamTokenizer.TT_WORD)
	  LineParser.exception(st,"Expected Number(1)");
	return Integer.parseInt(st.sval);
      } 
      catch (IOException e) {
	LineParser.ioexception(e);

      }
      catch (NumberFormatException e) {
	LineParser.exception(st,"Expected number(2)");
      }
      return Integer.MAX_VALUE;
  }

  public static String getIntAsString(StreamTokenizer st)
    throws LineParserException {
      try {
	if (st.nextToken() != StreamTokenizer.TT_WORD)
	  LineParser.exception(st,"Expected Number(1)");
	Integer.decode(st.sval);
	return st.sval;
      } 
      catch (IOException e) {
	LineParser.ioexception(e);
      }
      catch (NumberFormatException e) {
	LineParser.exception(st,"Expected number(2)");
      }
      return Integer.toString(Integer.MAX_VALUE);
  }

  public static String getString(StreamTokenizer st,boolean casesens)
    throws LineParserException {
      try {
	int tok = st.nextToken();
	if (tok != StreamTokenizer.TT_WORD && tok != '"')
	  LineParser.exception(st,"Expected String");
	return new String(casesens ? st.sval : st.sval.toLowerCase());
      } catch (IOException e) {
	LineParser.ioexception(e);
      }
      return null;
  }

  public static double getDouble(StreamTokenizer st)
    throws LineParserException {
      try {
	if (st.nextToken() != StreamTokenizer.TT_WORD)
	  LineParser.exception(st,"Expected Number(1)");
	return Double.valueOf(st.sval).doubleValue();
      } 
      catch (IOException e) {
	LineParser.ioexception(e);
      }
      catch (NumberFormatException e) {
	LineParser.exception(st,"Expected Number(2)");
      }
      return Double.NaN;
  }

  public static String getDoubleAsString(StreamTokenizer st)
    throws LineParserException {
      try {
	if (st.nextToken() != StreamTokenizer.TT_WORD)
	  LineParser.exception(st,"Expected Number(1)");
	Double.valueOf(st.sval);
	return st.sval;
      } 
      catch (IOException e) {
	LineParser.ioexception(e);
      }
      catch (NumberFormatException e) {
	LineParser.exception(st,"Expected Number(2)");
      }
      return Double.toString(Double.NaN);
  }

  public static int getKeyword(StreamTokenizer st,String[] kwds,boolean casesens)
    throws LineParserException {
      int i;
      String str = getString(st,casesens);
      for (i=0; i<kwds.length; i++)
	if (str.equals(kwds[i]))
	  return i+1;
      LineParser.exception(st,"Expected keyword, found "+str);
      return -1;
  }

  public static void skipToEol(StreamTokenizer st) 
    throws LineParserException {
      try {
	for(;;) {
	  int tok = st.nextToken();
	  if (tok == StreamTokenizer.TT_EOL ||
	      tok == StreamTokenizer.TT_EOF) {
	    st.pushBack();
	    return;
	  }
	}
      }
      catch(IOException e) {
	LineParser.ioexception(e);
      }
  }
  
  public static void exception(StreamTokenizer st,String msg) 
    throws LineParserException {
      int lineno = st != null ? st.lineno() : 0;
      throw new LineParserException("Line "+lineno+": "+msg);
  }
 
  public static void ioexception(IOException e) 
    throws LineParserException {
      throw new LineParserException("I/O Exception: "+e);
  }
 
  public LineParser(String filename,Object obj,Hashtable cmds) 
    throws LineParserException,FileNotFoundException {
      this(new StreamTokenizer(new FileReader(filename)),obj,cmds);
  }

  public LineParser(StringReader string,Object obj,Hashtable cmds) 
    throws LineParserException {
      this(new StreamTokenizer(string),obj,cmds);
  }

  public LineParser(StreamTokenizer st,Object obj,Hashtable cmds) 
    throws LineParserException {
      try {
      	st.resetSyntax();
	st.eolIsSignificant(true);
	st.commentChar('#');
	st.slashStarComments(true);
	st.slashSlashComments(true);
	st.whitespaceChars(0,' ');
	st.quoteChar('"');
	st.wordChars('!','!');
	st.wordChars('$','~');
	
	int tok;
	
	while ((tok = st.nextToken()) != StreamTokenizer.TT_EOF) {
          //System.out.println("Lineparser " + st);
	  if (tok == StreamTokenizer.TT_WORD) {
	    String keywd = st.sval.toLowerCase();
            //System.out.println("Lineparser " + keywd);
	    LineParse e = (LineParse)cmds.get(keywd);
	    if (e == null)
	      exception(st,"Invalid token: "+keywd);
	    else if (e.read(st,obj) == false)
	      break;
	  } else if (tok== StreamTokenizer.TT_EOL)
	    continue;
	  else
	    exception(st,"Invalid tokentype "+tok);
	  
	  // expect at this point a parsed line
	  tok = st.nextToken();
	  if (tok == StreamTokenizer.TT_EOF)
	    break;
	  else if (tok == StreamTokenizer.TT_EOL)
	    continue;
          else  // Waiting for end of line, ignoring all comments!
	      {
               while(tok != StreamTokenizer.TT_EOL) tok = st.nextToken();
               continue;
              }
	  //else
	  //  exception(st,"Expected EOL");
	}
      }
      catch(IOException e) {
	ioexception(e);
      }
  }
}
