// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;

import java.awt.*;
import java.awt.geom.*;

/**
  * This tag stores a Map Object.
  * @author Holger Veit 
  * @version $Id: MapObjTag.java,v 1.2 2001/06/21 14:57:18 veit Exp $
  */
public abstract class MapObjTag extends FTATag 
  implements DynObjectI,Constants {

    protected Shape shape = null;
    protected static int objcount = 0;
    protected String name = null;
    protected Color shcolor;
    protected int shwidth;

    /**
      * public access methods
      */
    public Point2D getP1() { return null; }
    public Point2D getP2() { return null; }
    public Point2D getP3() { return null; }
    public Point2D getP4() { return null; }
    public double getRadius() { return 0.0; }
    public abstract int getObjType();

    public PathIterator getPoly() {
      return shape.getPathIterator(null);
    }

    public boolean intersects(Shape s) {
      return s != null && s.intersects(getBounds());
    }

    public Shape getShape() {
      return shape;
    }

    public Rectangle2D getBounds() {
      return shape.getBounds2D();
    }

    /**
      * Interface DynObjectI:
      */
    public void paint(Graphics g,int flags) {
      Graphics2D g2 = (Graphics2D)g;
      g2.setStroke(new BasicStroke(shwidth));
      g2.setColor(shcolor);
      g2.draw(getShape());

      Point2D p = getP1();
      if ((flags & DRAWNAME) != 0 && p != null) {
	g2.drawString(getName(),(int)p.getX()+5,(int)p.getY()+5);
      }
    }

    //public Point getSector() { return sector; }
    //public Point[] getNormVec() { return normVec; }
    
    /**
      * number of tags of this type
      * @return the approx. number of tags to be expected during a flip cycle
      */
    public int getCapacityHint() {
      return 100;
    }
    
    /**
      * all tags derived from this have the store type "MapObj"
      * @return the storetype for this tag
      */
    public String getStoreType() {
      return "jfta.demo.simrob.MapObjTag";
    }
    
    /**
      * mandatory empty constructor
      */
    public MapObjTag() {
      this(null,Color.black,1);
    }
    
    /**
      * custom constructor
      */
    protected MapObjTag(String n,Color col,int width) {
      super();
      name = n;
      shcolor = col;
      shwidth = width;
    }

    /**
      * Interface DynObjectI:
      */
    public String getName() {
      return name;
    }

    /**
      * Interface DynObjectI:
      */
    public boolean equals(DynObjectI obj) {
      String n = obj.getName();
      return name != null && n != null &&
	n.equals(name);
    }

    /**
      * Interface DynObjectI:
      */
    public void modify(ModObjectI mobj) {
      // do nothing
    }
}
