// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;

import java.awt.geom.*;
import java.math.*;

/**
  * SensLayoutTag is used by SimLaserScanner 
  */
public class SensLayoutTag extends FTATag {

  private int VehNumber;
  private int SensNumber;
  private String Type;
  private Point2D.Double Pos;
  private double Direction;
  private double MaxRange;
  private double MinRange;
  private double LeftAngle;
  private double RightAngle;
  private double AngleStep;
  private double HasError;
 

  /**
    * get the number of the vehicle
    * @return the vehicle number
    */
  public int getVehNumber() {
    return VehNumber;
  }
  
  /**
   * @return the sensor number
   */
  public int getSensNumber() { 
    return SensNumber;
  }
  
  /*
   * @return the sensor type
   */
  public String getSensType()
  {
    return Type; 
  }
  
  /** 
    * @return the Vehicle Position
    */
  public Point2D.Double getPos() {
    return Pos;
  }

  /**
    * @return the direction of the sensor 
    */
  public double getDirection() {
    return Direction;
  }

  /**
    * @return the maximum range
    */
  public double getMaxRange() {
    return MaxRange;
  }

  /**
    * @return the minimum range
    */
  public double getMinRange() {
    return MinRange;
  }

  /**
    * @return the left angle
    */
  public double getLeftAngle() {
    return LeftAngle;
  }

  /**
    * @return the right angle
    */
  public double getRightAngle() {
    return RightAngle;
  }

  /**
    * @return the angle step
    */
  public double getAngleStep()  {
    return AngleStep;
  }
 
  /**
    * @return the error
    */
  public double getError()  {
    return HasError;
  } 


  /**
    * constructor
    */
  public SensLayoutTag() {
    super();
  }

  /**
    * mandatory init method
    * @param args the argument string used to initialize the tag
    * @exception FTAArgumentException Thrown in case of an invalid or missing
    *   argument
    */
  public void init(String[] args) throws FTAArgumentException {
      if (args.length != 12)
	  throw new FTAArgumentException("Requires 12 args");
      
      int i = 0;
      
      try {
	  VehNumber  = Integer.valueOf(args[i]).intValue();
	  SensNumber = Integer.valueOf(args[i++]).intValue();
	  
	  Type = args[i++];
	  
	  Pos = new Point2D.Double();
	  Pos.x = Double.valueOf(args[i++]).doubleValue();
	  Pos.y = Double.valueOf(args[i++]).doubleValue();
       
	  Direction = Double.valueOf(args[i++]).doubleValue();
	  MaxRange = Double.valueOf(args[i++]).doubleValue();
	  MinRange = Double.valueOf(args[i++]).doubleValue();
	  LeftAngle = Double.valueOf(args[i++]).doubleValue();
	  RightAngle = Double.valueOf(args[i++]).doubleValue();
	  AngleStep = Double.valueOf(args[i++]).doubleValue();
	  HasError  =  Double.valueOf(args[i++]).doubleValue();
      }
      catch (Exception e) {
	  throw new FTAArgumentException("Invalid argument #" + i);
      }  
  }
}




















