// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.actors.*;
import jfta.exception.*;
import jfta.tags.*;
import java.awt.geom.Point2D;

/**
 * This actor simulates the laser scanner.
 */
public class SimLaserScanner extends SimObject {
    
  private FTACallbackManager cbm;

  private int NOLScanner;
  
  private int posX,posY;
  private double orient;
     
  private SensorBeam LScannerBeam;
  
  private boolean getNewVehicle() throws FTAEvalActorException {
    int numScanners;
    FTATagIterator it = getTagIterator(configTB,vconfpattern); 
    while (it.hasNext()) {
      Object obj = it.next();
      if (obj instanceof VConfVehiclelocTag) {
	VConfVehiclelocTag tag = (VConfVehiclelocTag)obj;
	
	numScanners = tag.getNOLaserScanner();
	// assert(iNumS <= 1) - check probably not needed 
	if (numScanners > 1)
	  throw new FTAEvalActorException("number of scanners > 1");
	NOLScanner = numScanners; 
	// returns immediately - diff. to c++ Version 
	return true;
      }
    }
    return false;
  }

  private void getVehiclePos() throws FTAEvalActorException {
    FTATagIterator it = getTagIterator(locTB,vconfpattern);
    while (it.hasNext()) {
      VConfVehiclelocTag tag = (VConfVehiclelocTag)it.next();
      
      posX = tag.getX();
      posY = tag.getY();
      orient = tag.getOrient();
    }
  }
    
  private void getSensorLayout() throws FTAEvalActorException {
    int sensornr;
    int amount = 0;
    FTATagIterator it = getTagIterator(configTB,slaypattern);
    while (it.hasNext()) {
      Object obj = it.next();
      if (obj instanceof SensLayoutTag) {
	SensLayoutTag tag = (SensLayoutTag)obj;
	
	// getType --> getSensType
	// sensornr should be 0 and amount should be 1
	if (tag.getSensType().equals("KST_lscan")) {
	  sensornr = tag.getSensNumber();
	  if (sensornr != 0)
	    throw new FTAEvalActorException("SensorNr is not 0");
	  if (++amount > 1)
	    throw new FTAEvalActorException("#sensors > 1");
	  // LScannerBeam is int ?? SensorBeam ??
	  LScannerBeam = new SensorBeam(sensornr,tag);
	  // return; ?? amount not needed
	}
      }
    }
  }
    
  /**
    * Laserscanner init. Returns an exception if no laserscanners were 
    * found.  
    */
  private void initLaserScanner() throws FTAEvalActorException {
    if (getNewVehicle()) {
      getSensorLayout();
      if (NOLScanner == 0) 
	throw new FTAEvalActorException("no laserscanner found");
    }
  }
  
  private class LocCallback extends FTATBCallback {
    public LocCallback(FTAActor ac) {
      super(ac,locTB);
    }
    public int eval2() throws FTAEvalActorException {
      // immediately give up eval run
      return FTAActor.YIELD;
    }
  }

  private class VConfCallback extends FTATBCallback {
    public VConfCallback(FTAActor ac) {
      super(ac,configTB);
    }
    public int eval2() throws FTAEvalActorException {
      SimLaserScanner slc = (SimLaserScanner)ac;
      slc.initLaserScanner();
      return FTAActor.KEEPALIVE;
    }
  }

  /**
    * eval method
    */
  public int eval() throws FTAEvalActorException {
    int ret = cbm.eval();
    if (ret != FTAActor.KEEPALIVE)
      return ret;

    getVehiclePos();
    // LScannerBeam.calcSensorValue(...);
    // putTag (SensorTB,senstag);
    return FTAActor.KEEPALIVE;
  }
    
  public SimLaserScanner(String veh,int num) {
    super("SimLaserScanner",veh);

    NOLScanner = 0;

    try {
      initLaserScanner();

      cbm = new FTACallbackManager();
      cbm.add(new MsgCallback(this));
      cbm.add(new LocCallback(this));
      cbm.add(new TimediffCallback(this,0.2));
      cbm.add(new MapCallback(this));
      cbm.add(new VConfCallback(this));
      
      userMessage(WhoAmI, "START");
    }
    catch (FTAEvalActorException e) {
      userMessage(WhoAmI,"START Failed: "+e.getMessage());
    }
  }
}
