// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;

import java.awt.geom.*;
import java.math.*;
import java.util.*;

/**
 * This actor simulates the motor for the vehicle with name _myname
 * This means movement, odometry and collisions!
 * Like in the C++-Version it does NOT check collisions between
 * different vehicles!!!!
 * PlaySound at collision Not Implemented 
 * @author Frank Schoenherr, Andre Schmitz, Holger Veit
 * @version $Id: SimMotor.java,v 1.2 2001/06/21 14:57:19 veit Exp $
 */
public class SimMotor extends SimObject {

    private FTACallbackManager cbm;

    /**  
     * Some internal states
     */
    private boolean SpeedSelected = false;
    private boolean Lifted = false;
    private boolean SimRunning = true;
    private boolean RealTime = true;
    private boolean LayoutLoaded = false;
    private boolean Collision = false;
    private boolean PlaySound = false;
    private double DeltaT; // Time in sec. since last actor cycle 
    private long LastTime; // Timestamp of the last actor cycle in milliseconds
 
    /**
     *  save the vehicle configuration
     */
  
    private int NOMotors=0;  
    private double Width;
    private double Length;
    private Point2D.Double Pos;
    private Point2D.Double newPos;
    private double posX,posY;
    private double Orient;
    private String VehType;
    private double[] dTicksPerPWM,dTicksPerMM;
    private double[] MaxTickAccelMot;
    private short[] selMotorPWM,selMotorSpeed;
    private short[] MotorPWM,MotorSpeed;
    private short[] TicksPerSec;
    private long[] TickCount,LastCount;
    private double[] dMMPerTick,dMMPerPWM;

    // vehicle motor characteristic
    private Vector mot;

    // layout of this vehicle loaded from VConfTB?
    boolean foundnew = false;
  

    /**
     * motor access
     */
    private int getTicksPTurn(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getTicksPTurn();
    }
    private int getMaxTicksPSec(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getMaxTicksPSec();
    }
    private double getTickError(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getTickError();
    }
    private double getX(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getX();
    }
    private double getY(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getY();
    }
    private double getDirection(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getDirection();
    }
    private double getWheelDiam(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getWheelDiam();
    }
    private double getMaxSpeed(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getMaxSpeed();
    }
    private double getMinSpeed(int motnum) {
	return ((VConfMotLayoutTag)mot.get(motnum)).getMinSpeed();
    }

    /**
     * get the motor layout of this vehicle from the configTB
     */
    private class MotorCallback extends FTATBCallback {
	public MotorCallback(FTAActor ac,String tb) {
	    super(ac,tb);
	}

	public int eval2() throws FTAEvalActorException {
	    System.out.println("Get MotorCallback tag"); 
	    FTATagIterator iter = getTagIterator(configTB,mlaypattern);
	    int mnum;

	    // read data (already have the NOMotors from the layout tag
	    VConfMotLayoutTag tag = null;  
	    Vector mot = new Vector(NOMotors);
	    dTicksPerPWM = new double[NOMotors];
	    MaxTickAccelMot = new double[NOMotors];
	    selMotorPWM = new short[NOMotors];
	    selMotorSpeed = new short[NOMotors];
	    MotorPWM = new short[NOMotors];
	    MotorSpeed = new short[NOMotors];
	    TicksPerSec = new short[NOMotors];
	    TickCount = new long[NOMotors];
	    LastCount = new long[NOMotors];
	    dTicksPerMM = new double[NOMotors];
	    dMMPerTick = new double[NOMotors];
	    dMMPerPWM = new double[NOMotors];
      
	    while (iter.hasNext()) {
		tag = (VConfMotLayoutTag)iter.next();
		System.out.println("found one tag"); 
		mnum = tag.getNumber();
		if (mnum > NOMotors)
		    throw new FTAEvalActorException(WhoAmI +
						    ": invalid motor number in tag: " +
						    mnum);
		mot.add(mnum,tag);
	
		// A. Schmitz: just a random number !!! change
		dTicksPerPWM[mnum] = 23.0; 
	
		// A. Schmitz: 800 Ticks per sec max. acceleration
		MaxTickAccelMot[mnum] = 800.0;
		// reset values
		selMotorPWM[mnum] = selMotorSpeed[mnum] = MotorPWM[mnum] = 0; 
		MotorSpeed[mnum]= TicksPerSec[mnum] = 0;
		TickCount[mnum] = LastCount[mnum] = 0; 
	
		dTicksPerMM[mnum] = getTicksPTurn(mnum) / 
		    (Math.PI * getWheelDiam(mnum));
		dMMPerTick [mnum] = 1/dTicksPerMM[mnum];
		dMMPerPWM  [mnum] = dMMPerTick[mnum] * dTicksPerPWM[mnum];  
		LayoutLoaded = true;
		break;
	    }
	    return FTAActor.KEEPALIVE;
	}
    }
  
    /**
     * get the layout of this vehicle from the configTB
     */
    private class ConfigCallback extends FTATBCallback {
	public ConfigCallback(FTAActor ac,String tb) {
	    super(ac,tb);
	}
	public int eval2() throws FTAEvalActorException {
	    // read new tag
	    System.out.println("Get ConfigCallback"); 
	    FTATagIterator iter = getTagIterator(configTB,vlocpattern);
	    while (iter.hasNext()) {
		System.out.println("found one tag"); 
		VConfVehiclelocTag tag = (VConfVehiclelocTag)iter.next();
		NOMotors = tag.getNOMotors();
		if (NOMotors==0)
		    throw new FTAEvalActorException(WhoAmI + 
						    ": no motors found in vehicle");
		System.out.println(NOMotors); 
		Width = tag.getWidth();
		Length = tag.getLength();
		posX = tag.getX();
		posY = tag.getY();
		Orient = tag.getOrient();
		VehType = new String(tag.getVType());
		foundnew = true; // foundnew is a global variable now!!!
	    }
	    return KEEPALIVE;
	}
    }

    /**
     * get the control commands for this vehicle from the CmdTB
     */
    private class ControlCallback extends FTATBCallback {
	public ControlCallback(FTAActor ac,String tb) {
	    super(ac,tb);
	}
	public int eval2() throws FTAEvalActorException {
	    // read new tag
	    System.out.println("Get ControlCallback tag");  
	    int cmdtype; 
	    int motnum;  
	    VehCmdTag tag=null;
	    FTATagIterator iter = getTagIterator(cmdTB,vehpattern);
	    while (iter.hasNext()) {
		tag = (VehCmdTag) iter.next();  
		cmdtype = tag.getCmd(); 
		if(cmdtype==VehCmdTag.VC_INVALID){   
		    userMessage("SimMotor"+vehname+".getCommands()", "The received command is INVALID!");
		} 
		if(cmdtype==VehCmdTag.VC_RESETODOM){ 
		    for(motnum=0; motnum < NOMotors; motnum++){
			LastCount[motnum] = TickCount[motnum];
			TickCount[motnum] = 0;
		    }
		}
		if(cmdtype==VehCmdTag.VC_SETVEHSPEED){ 
		    userMessage("SimMotor"+vehname+".getCommands()", "The SetVehSpeed-Command is not yet implemented!!!");
		}
		if(cmdtype==VehCmdTag.VC_SETMOTSPEED){  
		    motnum  = (int) ((VehCmdSetMotSpeedTag)tag).getMot();
		    System.out.println(motnum+" "+NOMotors); 
		    selMotorSpeed[motnum] = (short) ((VehCmdSetMotSpeedTag)tag).getSpeed();
		    SpeedSelected = true;
		}   
		if(cmdtype==VehCmdTag.VC_SETMOTPWM){
		    motnum  = (int) ((VehCmdSetMotPWMTag)tag).getMot();
		    selMotorPWM[motnum] = (short) ((VehCmdSetMotPWMTag)tag).getPWM();
		    SpeedSelected = false;
		} 
	    }  
	    return KEEPALIVE;    
	}
    }

    /**
     * get the messages for this vehicle from the msgTB
     */
    private class MessageCallback extends FTATBCallback {
	public MessageCallback(FTAActor ac,String tb) {
	    super(ac,tb);  
	}
	public int eval2() throws FTAEvalActorException {
	    // read new tag   
	    FTATagIterator iter = getTagIterator(msgTB,msgpattern);
	    while (iter.hasNext()) {
		MsgTag tag = (MsgTag)iter.next(); 
		int type = tag.getMsgType();
		if (type == MsgTag.TERMINATE) {
		    userMessage(WhoAmI,"EXITING...");
		    return TERMINATE;
		}if (type == MsgTag.REMVEHICLE)
		    System.out.println(vehname.equals(((MsgRemVehicleTag)tag).getVeh()));
		if ((type == MsgTag.REMVEHICLE) && (vehname.equals(((MsgRemVehicleTag)tag).getVeh()))){
		    userMessage(WhoAmI,"EXITING...");
		    return TERMINATE;
		}
	    }
	    return KEEPALIVE;
	}
    }

   
    /**
     * mandatory eval method
     * @return TERMINATE if the actor terminates after this
     *   routine has been called, KEEPALIVE to have the actor
     *   be rescheduled in the next tick cycle.
     * @exception FTAEvalActorException thrown in case a fatal error occurred
     */
    public int eval() throws FTAEvalActorException { 
	return cbm.eval();
    }
  
    /**
     * mandatory init method
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incorrect for this type of actor
     * @return returns true, if the initialization was okay, false if there
     *   has been a problem not requiring an <code>FTAArgumentException</code>
     */
    public void init(String[] args) throws FTAArgumentException {
	/*
	 * do final init
	 */
	initbase("SimMotor",getName());
    
	cbm = new FTACallbackManager();
	cbm.add(new SimMotor.MapCallback(this)); // from super class
	cbm.add(new ConfigCallback(this,configTB));
	cbm.add(new MotorCallback(this,configTB));
	//cbm.add(new ControlCallback(this,cmdTB));
	cbm.add(new MessageCallback(this,msgTB));
    
	/*
	 * notify start
	 */
	userMessage(WhoAmI," START");
	//try{flipOrder(cmdTB);}catch(FTAUnknownObjectException e){};
	LastTime = System.currentTimeMillis();
    }
  
    /**
     * basic empty constructor
     */
    public SimMotor() {
	super();
    }
}

