// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.demo.simrob;

import jfta.actors.*;
import jfta.exception.*;
import jfta.tags.*;
import java.util.*;


public class SimObject extends FTAActor implements FTAActorI,Constants {
   
  /**
    * common objects
    */
  protected String configTB;
  protected String cmdTB;
  protected String speedTB;
  protected String sensorTB;
  
  /**
    *  should be initialized by the overwritten init method
    */
  protected String vehname = null;
  protected String WhoAmI = null;
  
  // saves the local copy of the EnviTB = the map 
  protected Vector map;
  int noMapObjects;
  
  public int eval() throws FTAEvalActorException {
    throw new FTAEvalActorException("SimObject: this method should be overwritten.");
  }

  protected void initbase(String who,String veh) {
    vehname = veh;
    WhoAmI = who + "_" + veh;

    sensorTB = RobotLoader.getNamedObject(veh,SENSORTB);
    speedTB = RobotLoader.getNamedObject(veh,SPEEDTB);
    cmdTB =  RobotLoader.getNamedObject(veh,CMDTB);
    configTB =  RobotLoader.getNamedObject(veh,CONFTB);

    noMapObjects = 0;
  }

  public SimObject() {
    super();
  }

  public SimObject(String who,String veh) {
    super();
    initbase(who,veh);
  } 

  protected class MsgCallback extends FTATBCallback {
    public MsgCallback(FTAActor ac) {
      super(ac,msgTB);
    }
    public int eval2() throws FTAEvalActorException {
      FTATagIterator it = getTagIterator(tb,msgpattern);
      while (it.hasNext()) {
	Object tag = it.next();
	boolean term = false;

	if (tag instanceof MsgTerminateTag)
	  term = true;
	else if (tag instanceof MsgRemVehicleTag) {
	  MsgRemVehicleTag vt = (MsgRemVehicleTag)tag;
	  if (vehname.equals(vt.getVeh()))
	    term = true;
	}

	if (term) {
	  userMessage(WhoAmI,"EXITING...");
	  return FTAActor.TERMINATE;
	}
      }
      return FTAActor.KEEPALIVE;
    }
  }

  /**
    * Everytime this class is involved it calculates the time difference 
    * between the last call and the actual call, and compares it with a given
    * delta.
    * @PARAM delta expected time difference between two calls of this method
    */
  protected class TimediffCallback extends FTACallback {
    private double delta;
    private double lastActivation;
    public TimediffCallback(FTAActor ac, double delta) {
      super(ac);
      this.delta = delta;
      lastActivation = 0.0;
    }
    public int eval() throws FTAEvalActorException {
      double curTime = (System.currentTimeMillis() / 1000.0);
      double deltaT  = curTime - lastActivation;
      if (deltaT < delta)
	return FTAActor.YIELD;
      lastActivation = curTime;
      return eval2();
    }
    public int eval2() throws FTAEvalActorException {
      return FTAActor.KEEPALIVE;
    }
  }

  /** 
    * The MapCallback class stores the map in an array.
    * The return value of getTagCount and the amount of tags in the 
    * TagIterator may differ, so I used a temp. vector for intermediate 
    * storage of the tags.
    * @RETURN true if no exception was thrown 
    */
  protected class MapCallback extends FTATBCallback {
    public MapCallback(FTAActor ac) {
      super(ac,enviTB);
    }
    public int eval2() throws FTAEvalActorException {
      Vector map = new Vector();

      FTATagIterator it = getTagIterator(tb,mappattern);
      while (it.hasNext()) {
	map.add(it.next());
      }
      noMapObjects = map.size();
      return KEEPALIVE;
    }
  }
}
