// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: SimServer.java,v 1.2 2001/06/21 14:57:20 veit Exp $
// In this Beta-version all TB and Actors are started on the same host=thishost
// Should be changed to different hosts for a better performace
// So the ADD command could be: add filename hostname

package jfta.demo.simrob;

import java.util.*;
import java.io.*;
import java.rmi.*;
import java.awt.geom.*;   

import jfta.exception.*;
import jfta.tags.*;
import jfta.actors.*;
import jfta.base.*;
import jfta.system.*;    

/**
  * This actor does the complete command dispatch of the simrob simulator
  * @author Holger Veit, Andre Schmitz, Frank Schoenherr
  * @version $Id: SimServer.java,v 1.2 2001/06/21 14:57:20 veit Exp $
  */

public class SimServer extends FTAActor implements FTAActorI,Constants {

  private final static String WhoAmI = "SimServer";

  /**
    * the internal RMI communication object
    */
  private SimServerRemote _simcom = null;

  /**
    * this actor may not be instantiated multiple times
    */
  private static boolean _once = false;

  /**
    * instance name
    */
  private String _myinstname = null;

  /**
    * command list
    */
  private Hashtable _mycmds = null;

  /**
    * system exit
    */
 
  private boolean terminate = false;
  
  /*
   * private command handlers
   */

  /**
    * handle the LOADMAP command
    * syntax: LOADMAP "filename"
    */
  private class LoadMapCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	// get the actor object
	FTAActor ac = (FTAActor)obj;

	// read the filename argument from the line
	String filename = LineParser.getString(st,true);
	try {
	  // submit the tag to the msg tag board
	  ac.putTag(msgTB, new MsgLoadMapTag(filename));
	  
	  // explictly flip the tb
	  ac.flipOrder(msgTB);
	}
	catch (FTAEvalActorException e) {
	  // e.printStackTrace();
	  LineParser.exception(st,"LoadMap: " + e.getMessage());
	  return false;
	}
	userMessage(WhoAmI,"Map " + filename + " loaded.");
	return true;
    }
  }

  /**
    * handle the ADD command
    * syntax: ADD "name" "filename"
    */
  private class AddVehicleCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	// get the actor object
	FTAActor ac = (FTAActor)obj;
	  
	// read the vehicle name and filename from the line
	String veh = LineParser.getString(st,true);
	String file = LineParser.getString(st,true);

	try {
	  // submit the tag to the msg tag board
	  ac.putTag(msgTB, new MsgAddVehTag(veh,file));
	  
	  // explictly flip the tb
	  ac.flipOrder(msgTB);
	}
	catch(FTAEvalActorException e) {
	  // e.printStackTrace();
	  LineParser.exception(st,"AddVehicle: " + e.getMessage());
	  return false;
	}
	userMessage(WhoAmI,"Veh " + veh + " loaded.");
	return true;
    }
  }

  private static int account = 1;

  /**
    * handle the LOADACTOR command
    * syntax: LOADACTOR "actorname"
    */
  private class LoadActorCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
	try {
	  // get the actor object
	  FTAActor ac = (FTAActor)obj;

	  // read the actorname argument from the line
	  String actype = LineParser.getString(st,true);
	  
	  // create a unique name
	  String acname = "ACTOR" + account;
	  account++;

	  // create an actor
	  ac.addActor(acname,actype,(String[])null);
	  userMessage(WhoAmI,"Created new actor " + acname);
	}
	catch(FTAEvalActorException e) {
	  //e.printStackTrace();
	  LineParser.exception(st,"LoadActor: addActor argument error");
	  return false;
	}
	return true;
    }
  }

  /**
    * handle the BEHAVIOR command
    * syntax: BEHAVIOR name full.java.class.name
    * The Actor "Behavior"+name is created on the same host
    * and the same ensemble as the SimServer (TO CHANGE LATER!!!!) 
    */
  private class BehaviorCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {        
        String[] args = new String[2];
        // get the Veh name
        args[0] = LineParser.getString(st,false);
        // get the classname of the behavior
        args[1] = LineParser.getString(st,true);
        String CmdTB = "CommandTB_" + args[0];
        String SensorTB = "SensorTB_" + args[0];
        String SpeedTB = "SpeedTB_" + args[0];
        try {
	  flipOrder(CmdTB);
	  flipOrder(SensorTB);
	  flipOrder(SpeedTB);
	}
	catch(FTAUnknownObjectException e) {
	  userMessage(WhoAmI, "The Tagboards for the vehicle " + 
		      args[0] + " do not exist!!"); 
          return false;
	}
	try {
	  String BehaviorAc = "Behavior_" + args[0];
	  //String[] actargs = new String [1];
	  //actargs[0] = SensorTB;
	  //actargs[1] = SpeedTB;
	  //actargs[2] = CmdTB;
	  //actargs[3] = args[0]; 
	  addActor(BehaviorAc,args[1],args);
	}
	catch(FTAUnknownObjectException e) {
	  userMessage(WhoAmI, "The class " + 
		      args[1] + " does not exist!!"); 
          return false;
	}
	catch(FTAException e) {
	  userMessage(WhoAmI, "Could not create Actor: Behavior" +
		      args[0]);  
	  e.printStackTrace();  
          return false;
	}
        userMessage(WhoAmI,"Adding new behavior");
     
        return true;
    }
  }

  /**
    * handle the REMOVE command which will remove the vehicle 'name'
    * from the simulator 
    * syntax: REMOVE name
    */
  private class RemVehicleCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
        try {
            String[] args = new String[1];

            // get the Veh name
            String veh = LineParser.getString(st,false); 
	    
            // submit the Remove-tag to the msg tag board
	    putTag(msgTB, new MsgRemVehicleTag(veh));
	    
	    // explictly flip the Msg-TB
	    flipOrder(msgTB);
	}
	catch(FTAException e) {
	  LineParser.exception(st,"RemVehicle: puttag argument error");
	  return false;
	} 
        return true;
    }
  }

  /**
    * handle the OPENDOOR command to open or close a named door
    * manually
    * syntax: OPENDOOR name 0/1
    */
  private class OpenDoorCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
        try {
	  String[] args = new String[2];
	  
	  // get the door name
	  //args[0] = LineParser.getString(st); 
	  //args[] = LineParser.getIntAsString(st); 
	  
	  // submit the Remove-tag to the msg tag board
	  //putTag(msgTB, new...._remvtag, args);
	  
	  // explictly flip the Msg-TB
	  flipOrder(msgTB);
	  
	}
	catch (FTAException e) {
	  LineParser.exception(st,"OpenDoor: puttag argument error");
	  return false;
	} 
        return true;
    }
  }

 /**
    * handle the QUIT command
    * syntax: QUIT
    */
  private class QuitCmd extends Object implements LineParse {
    public boolean read(StreamTokenizer st,Object obj)
      throws LineParserException {
        try {
	  // submit the Terminate-tag to the msg tag board
	  putTag(msgTB, new MsgTerminateTag());
	  
	  // explictly flip the Msg-TB
	  flipOrder(msgTB);
	  terminate=true;
	}
	catch(FTAException e) {
	  LineParser.exception(st,"Quit: puttag argument error");
	}
        
        return false;
    }
  }

  /**
    * mandatory eval method
    * @return TERMINATE if the actor terminates after this
    *   routine has been called, KEEPALIVE to have the actor
    *   be rescheduled in the next tick cycle.
    * @exception FTAEvalActorException thrown in case a fatal error occurred
    */
  public int eval() throws FTAEvalActorException {

    if (terminate) { 
       userMessage(WhoAmI,"EXITING...");
       System.exit(0);      
    }
    String cmd = null;
    try {
      // while commands available
    if(_simcom.available())
	{
        while (_simcom.available()) {
          // read a command
          cmd = _simcom.getCmd();
          userMessage(WhoAmI,"Cmd received = \"" + cmd + "\"");
          LineParser p = new LineParser(new StringReader(cmd),this,_mycmds);
         }
        }
     else return FTAActor.KEEPALIVE;
       
    }
    catch (LineParserException e) {
      userMessage(WhoAmI,"Invalid command: \"" + cmd + "\"");
    }

    return FTAActor.KEEPALIVE;
}
  
  /**
    * mandatory init method
    * @param args list of arguments passed by <code>addActor</code> to the
    *   newly created actor. 
    * @exception FTAArgumentException to be thrown by the user code if some
    *   argument is incoreect for this type of actor
    * @return returns true, if the initialization was okay, false if there
    *   has been a problem not requiring an <code>FTAArgumentException</code>
    */
  public void init(String[] args) throws FTAArgumentException {

    // may only be created once
    if (_once)
      throw new FTAArgumentException("Cannot instantiate multiple copies!");

    // requires a single argument, the instance name that the communication
    // URL should use
    _myinstname = null;
    if (args.length == 1)
      _myinstname = args[0];

    // fill the command table
    _mycmds = new Hashtable();
    _mycmds.put("loadmap",new LoadMapCmd());
    _mycmds.put("loadactor",new LoadActorCmd());
    _mycmds.put("behavior",new BehaviorCmd()); 
    _mycmds.put("quit",new QuitCmd());
    _mycmds.put("add",new AddVehicleCmd());
    _mycmds.put("remove",new RemVehicleCmd());
    _mycmds.put("opendoor",new OpenDoorCmd());



    // create a remote server object for communication
    try {
      _simcom = new SimServerRemote(_myinstname);
      userMessage(WhoAmI,"Registered as URL=" + _simcom.getURL());
    }
    catch(RemoteException e) {
      e.printStackTrace();
      throw new FTAArgumentException("Could not setup SimServerRemote");
    }
    _once = true;
  }
  
  /**
    * mandatory empty constructor
    */
  public SimServer() {
    super();
  }
}
