// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTAEnsembleSub.java,v 1.14 2001/06/21 14:57:22 veit Exp $

package jfta.ensemble;

import java.util.*;
import java.net.*;

import jfta.base.*;
import jfta.actors.*;
import jfta.bootstrap.*;
import jfta.exception.*;
import jfta.system.*;
import jfta.tagboard.*;
import jfta.tags.*;
import jfta.http.*;

/**
  * EnsembleSub (ES) represents an ensemble on some remote host.
  * It contains a list of actors.
  * It is triggered by its EnsembleMain with worknow message.
  * When triggered, it will execute the eval method of all
  * contained actors, and process their return value
  * (KEEPALIVE or TERMINATE). When finished, it returns a
  * workdone message to its EM.
  * It accepts requests to create a new actor.
  * It can take over the role of being the EnsembleMain or
  * the Ensemble Backup
  */
public class FTAEnsembleSub extends Object
    implements FTAServletI,FTAEvalableI {

    private static FTAService _svc = FTAService.Instance();
  
    private long _cycletime;
    private long _millis;
    private long _precyc;
    private long _premillis;

    private FTASysSub _mysyssub;
    private String _mysysname;
    private String _mylogname;
    private String _myhostname;
    private Thread _work;
    private FTACommManager _comm;
    private int _deletecnt;
    private int _state;

    private int _myhostnum;
    private int _myobjnum;
    private FTAListenerI _listener;

    private long _profcnt;
    private long _profstart;
    private double _profmin;
    private double _profmax;
    private double _profavg;
    private double _proflast;

    private Object _runlock = new Object();
    private boolean _runprep;
    private boolean _runnow;
    private boolean _runack;
    private int _testflag;
    private int _testflag2;

    /**
     * @roseuid 383EB198D0A0
     */
    public FTAEnsembleSub(FTASysSub syssub,String enname,
			  int objnum,boolean start) {
	super();

	_mylogname = enname;
	_mysyssub = syssub;
	_myhostname = _svc.getFQDNname();
	_mysysname = _mysyssub.getName();
	_myhostnum = syssub.getObjnum();
	_myobjnum = objnum;
	_comm = _mysyssub.getCommManager();
	_deletecnt = -1;

	// profiler stuff
	_profcnt = 0;
	_profmin = 99999999999.9;
	_profmax = -9999999999.9;
	_profavg = 0.0;
	_proflast = 0.0;

	_cycletime = 0;
	_millis = 0;
	
	_listener = new ESListener();
	
	// finally notify ENMain that I am operable
	_comm.sendMsg(0,FTACreateENOfficeDoneMsg.get(_myhostnum,
						     _myobjnum));
    }

    private void uninstall() {
	// don't get messages any more
	_listener.uninstall(_comm);
	
	// tell sys office to discard self
	_mysyssub.removeEnsembleSub(_myobjnum,_mylogname);
    }

    /**
     * FTAEvalable API
     * This will be called at the end of a cycle of the FTASysSub.SysEvalThread.
     * It will check whether the EnsembleSub will be rescheduled in the
     * next cycle
     */
    public /*synchronized*/ int eval() throws FTAEvalException {
	synchronized(_runlock) {

	    boolean terminate = _deletecnt == 0;

	    // have to return an acknowledge?
	    if (_runack) {
		_testflag |= 1;
		// stop profile timer
		_proflast = System.currentTimeMillis() - _profstart;
		_profcnt++;
		_profavg += _proflast;
		_profmin = Math.min(_profmin,_proflast);
		_profmax = Math.max(_profmax,_proflast);
		
		// send acknowledge
		_comm.directAckCycle(_myhostnum,_myobjnum,_cycletime,terminate);
		_testflag |= 2;
	    }
	    
	    // manage the automatic delete in case no actor exists any longer
	    if (terminate) {
		uninstall();
		return TERMINATE;
	    }
	    
	    //FTADebug.log(_myobjnum,0,"Eval "+_mylogname+": p="+
	    // _runprep+" n="+_runnow+
	    // " a="+_runack);
	    
	    _testflag |= 4;
	    // update job queue
	    _runack = _runnow;
	    _runnow = _runprep;
	    _runprep = false;

	    // will we run a new cycle for all actors in the next round?
	    if (_runnow) {
		_testflag |= 8;
		// copy the new time stamp
		_cycletime = _precyc;
		_millis = _premillis;
		
		// start profile timer
		_profstart = System.currentTimeMillis();
		
		// decrement delete counter, this is supposed to be reset
		// by any active actor when calling getRunFlag(). If no actor
		// calls getRunFlag, this ensemble no longer has any actors,
		// and will thus be discarded after this now started cycle
		// will be acknowledged.
		if (_deletecnt > 0)
		    _deletecnt--;
	    }
	}
	    
	// will we terminate?
	if (_state==FTASysSub.FTACTLSHUTDOWN)
	    return TERMINATE;
	
	// we still exist
	return KEEPALIVE;
    }
    
    public boolean isEval() {
	return true;
    }

    /**
     * return true if an associated actor is supposed to run
     */
    public final boolean getRunFlag() {
	// reset the deletecounter
	_deletecnt = 20;

	// are we running now?
	return _runnow;
    }


    /**
     */
    public String toString() {
	return "ES_"+_mylogname;
    }

    /**
     */
    public String getName() {
	return _mylogname;
    }

    public int getObjnum() {
	return _myobjnum;
    }

    /**
     * profiling support
     */
    public final double getProfileInfo(int type) {
	switch (type) {
	case FTAActor.PROFMIN:
	    return _profmin;
	case FTAActor.PROFMAX:
	    return _profmax;
	case FTAActor.PROFAVG:
	    return _profcnt>0 ? (_profavg / _profcnt) : 0;
	case FTAActor.PROFLAST:
	    return _proflast;
	default:
	    return -1.0;
	}
    }

    public final long getEnsembleTime(int type) {
	return (type==FTAActor.ABSOLUTE) ? _millis : _cycletime;
    }

    /**
     * required to get a system sub for an actor
     * @roseuid 38B3C615F3DB
     */
    public final FTASysSub getSysSub() {
	return _mysyssub;
    }
    
    /******************************************************************/
    /* standard remote features */
    /******************************************************************/
    
    /**
     * this adapter manages registered messages
     */
    private class ESListener extends FTAListenerAdapter {
	public ESListener() {
	    super();
	    _comm.addListener(FTAMsg.STARTCYCLE,this,_myobjnum);
	    _comm.addListener(FTAMsg.CONTROL,this);
	}

	public void uninstall(FTACommManager comm) {
	    comm.removeListener(FTAMsg.STARTCYCLE,this);
	    comm.removeListener(FTAMsg.CONTROL,this);
	}
	
	// issued by SysMain to control operation
	public void notifyControl(int function, int code) {
	    if (FTADebug.CT)
		FTADebug.log(_myhostnum,_myobjnum,
			     " control: " + function);
	    _state = function;
	}

	// issued by ENMain to its ENSubs
	public void notifyStartCycle(long cyc,long millis) {
	    // ignore a duplicate
	    if (cyc == _cycletime) {
		_testflag2 |= 2;
		return;
	    }

	    _testflag = 0;
	    _testflag2 = 0;
	    if (FTADebug.WT)
		FTADebug.log(_myhostnum,_myobjnum,"worknow: "+cyc);
	    
	    /* set new time */
	    _precyc = cyc;
	    _premillis = millis;
	    
	    // request a run slice next time
	    synchronized(_runlock) {
		_runprep = true;
		_testflag2 |= 1;
	    }
	}
    }

    /******************************************************************/
    /* HTML interface */
    /******************************************************************/
    
    public FTAServletReply reply(FTAServletRequest req) {
	String head = req.getHead();
	if (head.equals("/") || head.equals("/index.html")) {
	    FTAHtmlFile f = new FTAHtmlFile("FTAEnsemble Office " + 
					    _mylogname);
	    f.h2("Properties");
	    f.beginUL();
	    f.propitem("Logical name",_mylogname);
	    f.propitem("Hostname",_myhostname+" (HostNum="+_myhostnum+")");
	    f.propitem("ObjectNum",_myobjnum+"");
	    f.propitem("Just Finished Tick Cycle",_cycletime+"");
	    f.propitem("System state",
		       _state + " ( "+FTASysSub.SysStates[_state] + ')');
	    f.propitem("Flags","a="+_runack+", n="+_runnow+", p="+_runprep);
	    f.propitem("TestFlag",_testflag+"");
	    f.propitem("TestFlag2",_testflag2+"");
	    Date now = new Date();
	    f.propitem("Time",now.toString());
	    f.endUL();
	    f.text("The following list is inaccurate, and links may even fail, when the system is running.");
	    f.h2("Local actors");
	    f.beginUL();
	    _mysyssub.listActors(f,this);
	    f.endUL();
	    return f.finish(getURL(4),"FTADaemon",getURL(2));
	} else {
	    head = head.substring(1);
	    FTAActor ac = _mysyssub.findActor(head);
	    if (ac != null) {
		req.shift();
		return ac.reply(req);
	    }
	    return FTAErrorReply.error404();
	} 
    }

    /**
     * locate an actor in a linked list
     * @param ll list to check
     * @param name name to search for
     */
    private FTAActor findActor(LinkedList ll,String name) {
	ListIterator it = ll.listIterator();
	while (it.hasNext()) {
	    FTAActor ac = (FTAActor)it.next();
	    if (ac.getName().equals(name))
		return ac;
	}
	return null;
    }

    private void listActors(FTAHtmlFile f,LinkedList ll) {
	ListIterator it = ll.listIterator();
	String myurl = getURL(0);
	while (it.hasNext()) {
	    FTAActor ac = (FTAActor)it.next();
	    String acname = ac.getName();
	    f.item(); f.anchor(myurl+acname+"/",acname);
	}
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public String getURL(int n) {
	switch (n) {
	case 4: /* its daemon */
	    return _mysyssub.getURL(4);
	case 2: /* its sysmain */
	    return _mysyssub.getURL(3);
	case 3: /* its parent */
	    return _mysyssub.getURL(1)+"$ENM$/"+_mylogname+"/";
	case 1: /* its syssub */
	    return _mysyssub.getURL(1);
	default: /* itself */
	    return _mysyssub.getURL(1)+"$ENS$/"+_mylogname+"/";
	}
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
