// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTACgiBinServlet.java,v 1.3 2001/06/21 14:57:23 veit Exp $

package jfta.http;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.base.*;
import jfta.exception.*;

/**
 * This servlet, though called CgiBinServlet, is a redirector for
 * a new level of files. It requires the name of a config file
 * like the servlet.cfg file as an argument. It uses the entries
 * in this file as a dispatcher of the next level of URL hierarchy.
 * Typically, it is used for the /cgi-bin hierarchy, where it calls
 * cgi-handler servlets, but other applications are possible.
 */
public class FTACgiBinServlet extends FTAServlet {

    protected Hashtable _cgiservlets;
    protected static FTAService _svc = FTAService.Instance();

    /**
     * mandatory (empty) constructor
     */
    public FTACgiBinServlet() {
	super();
    }

    /**
     * initialize servlet with arguments
     * @param server backlink to server
     * @param argv argument vector
     * @exception FTAArgumentException raised by the derived class init
     * method, when some required argument is incorrect
     */
    public void init(FTAHttpServer server,Vector argv)
	throws FTAArgumentException {
	if (argv.size() != 1)
	    throw new 
		FTAArgumentException("Invalid number of parameters");
	try {
	    String slpath = _svc.expandEnv((String)argv.get(0));
	    _cgiservlets = FTAServlet.load(server,slpath,_cgiservlets);
	}
	catch (IOException e) {
	    throw new FTAArgumentException(e.toString());
	}
    }

    /**
     * return an output file to the calling server
     * @param req request data
     * @return error code (no output of HTML) or 0 = OK
     */
    public FTAServletReply reply(FTAServletRequest req) {

	String tail = req.getTail();
	if (tail == null)
	    return FTAErrorReply.error404();

	return dispatchRequest(req,_cgiservlets);
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
