// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAErrorReply.java,v 1.3 2001/06/21 14:57:23 veit Exp $

package jfta.http;

import java.net.*;
import java.util.*;
import java.io.*;
import jfta.base.*;

/**
 * This class provides a standard error reply in case of an error.
 * Users can simply return a new instance of such an object in case
 * a situation not to be handled in their servlets occurs.
 */
public class FTAErrorReply extends FTAHtmlFile {

    /**
     * constructor
     * @param errcode the error code to be returned as a reply.
     */
    public FTAErrorReply(int errcode) {
	this(errcode,(String)null);
    }

    /**
     * constructor
     * @param errcode the error code to be returned as a reply.
     * @param info additional info
     */
    public FTAErrorReply(int errcode,String info) {
	super("Error "+errcode + ": " +
	      FTAContentHandler.getErrorMessage(errcode));
	_errorcode = errcode;
	p("Error " + errcode +
	  " occurred while processing page request");
	if (info != null)
	    text("Additional info: " + info);
	finish();
    }

    /**
     * generate an internal error for an exception
     * @param errcode error to return
     * @param exc exception that occurred
     */
    public FTAErrorReply(int errcode,Exception exc) {
	super("Error "+errcode + ": " +
	      FTAContentHandler.getErrorMessage(errcode));
	_errorcode = errcode;

	p("Error " + errcode +
	  " occurred while processing page request");

	ByteArrayOutputStream ba = new ByteArrayOutputStream(256);
	PrintStream es = new PrintStream(ba);
	exc.printStackTrace(es);
	p("Additional info:");
	String str = ba.toString();
	for (int i=0; i<str.length(); i++) {
	    char c = str.charAt(i);
	    if (c=='\n') br();
	    else add(c);
	}
	es.close();
	finish();
    }

    private static FTAErrorReply _error404 = null;

    /**
     * preconfigured error
     */
    public static FTAErrorReply error404() {
	if (_error404 == null)
	    _error404 = new FTAErrorReply(404);
	return _error404;
    }

    /**
     * preconfigured error
     */
    public static FTAErrorReply error500(String info) {
	return new FTAErrorReply(500,info);
    }
}
