// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAHtmlFile.java,v 1.5 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.net.*;
import java.util.*;

public class FTAHtmlFile extends FTAFile {

    public FTAHtmlFile() {
	super(1024,"text/html");
    }

    public FTAHtmlFile(String title) {
	this(title,true);
    }

    public FTAHtmlFile(String title,boolean hdr1) {
	super(1024,"text/html");
	add("<HTML><HEAD><TITLE>");
	add(title);
	add("</TITLE></HEAD>\r\n<BODY BGCOLOR=\"#ffffff\">\r\n");
	if (hdr1)
	    h1(title);
    }

    public FTAServletReply finish() {
	add("</BODY></HTML>\r\n\r\n");
	return this;
    }

    public void date() {
	Date d = new Date(System.currentTimeMillis());
	add(d.toString());
    }

    public FTAServletReply finish(String url,String linktext) {
	hr();
	add("Page generated: ");
	date(); br();
	add("Information: ");
	anchor(url,linktext);
	return finish();
    }

    public FTAServletReply finish(String url,String linktext,
				  String homeurl) {
	hr();
	add("Page generated: "); 
	date(); br();
	add("Information: ");
	anchor(url,linktext);
	add(" | ");
	anchor(homeurl,"Back to Top");
	return finish();
    }

    public void text(String text) {
	add(text);
    }

    public void markup(String text,String m) {
	add('<');
	add(m);
	add('>');
	add(text);
	add("</");
	add(m);
	add('>');
    }

    public void markup2(String text,String m) {
	markup(text,m);
	add("\r\n");
    }

    protected void single(String text,String s) {
	add(text);
	add(s);
    }

    public void br() {
	add("<BR>");
    }

    public void br(String text) {
	single(text,"<BR>");
    }

    public void hr() {
	add("<HR>");
    }

    public void p() {
	add("<P>\r\n");
    }

    public void p(String text) {
	single(text,"<P>\r\n");
    }

    public void h1(String text) {
	markup2(text,"H1");
    }

    public void h2(String text) {
	markup2(text,"H2");
    }

    public void h3(String text) {
	markup2(text,"H3");
    }

    public void h4(String text) {
	markup2(text,"H4");
    }

    public void h5(String text) {
	markup2(text,"H5");
    }

    public void h6(String text) {
	markup2(text,"H6");
    }

    public void b(String text) {
	markup(text,"B");
    }

    public void i(String text) {
	markup(text,"I");
    }

    public void u(String text) {
	markup(text,"U");
    }

    public void beginUL() {
	add("<UL>\r\n");
    }

    public void beginOL() {
	add("<OL>\r\n");
    }

    public void beginDL() {
	add("<DL>\r\n");
    }

    public void endUL() {
	add("</UL>\r\n");
    }

    public void endOL() {
	add("</OL>\r\n");
    }

    public void endDL() {
	add("</DL>\r\n");
    }

    public void item() {
	add("<LI>");
    }

    public void dt() {
	add("<DT>");
    }

    public void dt(String it) {
	dt();
	add(it);
    }

    public void dd() {
	add("<DD>");
    }

    public void dd(String it) {
	dd();
	add(it);
    }

    public void item(String it) {
	item();
	add(it);
    }

    public void propitem(String item,String property) {
	item(item+": ");
	b(property);
    }

    public void propitem(String item,int property) {
	item(item+": ");
	b(Integer.toString(property));
    }

    public void propitem(String item,long property) {
	item(item+": ");
	b(Long.toString(property));
    }

    public void propitem(String item,double property) {
	item(item+": ");
	b(Double.toString(property));
    }

    public void name(String name) {
	add("<A NAME=\"");
	add(name);
	add("\"></A>");
    }

    public void anchor(String url,String text) {
	add("<A HREF=\"");
	add(url);
	add("\">");
	add(text);
	add("</A>");
    }

    public void anchor(URL url,String text) {
	anchor(url.toString(),text);
    }

    public void img(String url) {
	img(url,"Image");
    }

    public void img(URL url) {
	img(url.toString(),"Image");
    }

    public void img(URL url,String alt) {
	img(url.toString(),alt);
    }

    public void img(String url,String alt) {
	add("<IMG SRC=\"");
	add(url);
	add("\" ALT=\"");
	add(alt);
	add("\">");
    }

    public void beginForm(String action) {
	beginForm("POST",action);
    }

    public void beginForm(String method,String action) {
	add("<FORM METHOD=\"");
	add(method);
	add("\" ACTION=\"");
	add(action);
	add("\">\r\n");
    }

    public void endForm() {
	add("</FORM>\r\n");
    }

    private void value(String val) {
	add("VALUE=\"");
	add(val);
	add("\">\r\n");
    }

    public void submitButton(String text) {
	add("<INPUT TYPE=\"SUBMIT\" ");
	value(text);
    }

    public void resetButton(String text) {
	add("<INPUT TYPE=\"RESET\" ");
	value(text);
    }

    public void inputField(int size,String name,String val) {
	add("<INPUT TYPE=\"TEXT\" NAME=\"");
	add(name);
	add("\" SIZE=\""+size+"\" ");
	value(val);
    }

    public void hiddenField(int size,String name,String val) {
	add("<INPUT TYPE=\"HIDDEN\" NAME=\"");
	add(name);
	add("\" SIZE=\"");
	add(Integer.toString(size));
	add("\" ");
	value(val);
    }

    public void passwordField(int size,String name,String val) {
	add("<INPUT TYPE=\"PASSWORD\" NAME=\"");
	add(name);
	add("\" SIZE=\"");
	add(Integer.toString(size));
	add("\" ");
	value(val);
    }

    public void beginTable() {
	add("<TABLE>");
    }

    public void endTable() {
	add("</TABLE>");
    }

    public void tableNewRow(String text) {
	add("<TR><TD>");
	add(text);
    }

    public void tableRow() {
	add("<TR>");
    }

    public void tableColumn() {
	add("<TD>");
    }

    public void tableColumn(String valign) {
	add("<TD VALIGN=\"");
	add(valign+"\">");
    }

    private void option(String text,boolean selected) {
	add("<OPTION");
	add(selected ? " SELECTED>" : ">");
	add(text+"\r\n");
    }

    public void selectBox(String name,
			  Hashtable list,
			  String first,
			  String selected) {
	add("<SELECT NAME=\"");
	add(name);
	add("\">\r\n");
	if (first != null)
	    option(first,selected.equals(first));
	Enumeration it = list.keys();
	while (it.hasMoreElements()) {
	    String key = (String)it.nextElement();
	    String val = (String)list.get(key);
	    option(val,selected.equals(key));
	}
	add("</SELECT>\r\n");
    }

    public void embedJava(String codebase,String name,int width,int height,
			  Hashtable params) {
	embedJava(codebase,name,width,height,params,"1.3");
    }

    public void embedJava(String codebase,String name,int width,int height,
			  Hashtable params,String version) {
	add("<EMBED TYPE=\"application/x-java-applet;version=");
	add(version+"\"\r\n WIDTH=\"");
	add(width+"\" HEIGHT=\"");
	add(height+"\"\r\n CODE=\"");
	add(name+"\" CODEBASE=\"");
	add(codebase+"\" PLUGINSPAGE=\"http://java.sun.com/products/plugin/");
	add(version+"/plugin-install.html\"\r\n");
	Enumeration it = params.keys();
	while (it.hasMoreElements()) {
	    String key = (String)it.nextElement();
	    Object val = params.get(key);
	    add(key+"=\""+val+"\"\r\n");
	}
	add("<NOEMBED>This page requires Java 2 1.3 support</NOEMBED>\r\n</EMBED>\r\n");
    }

    public void applet(String codebase,String name,int width,int height,
			  Hashtable params) {
	add("<APPLET WIDTH=\"");
	add(width+"\" HEIGHT=\"");
	add(height+"\"\r\n CODE=\"");
	add(name+"\" CODEBASE=\"");
	add(codebase);
	add("\" ALT=\"This page requires that JAVA is enabled\">\r\n");
	Enumeration it = params.keys();
	while (it.hasMoreElements()) {
	    String key = (String)it.nextElement();
	    Object val = params.get(key);
	    add("<PARAM NAME=\""+key+"\" VALUE=\""+val+"\">\r\n");
	}
	add("</APPLET>\r\n");
    }
}
