// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAHttpServer.java,v 1.5 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.util.*;
import java.net.*;
import java.io.*;
import jfta.base.*;
import jfta.bootstrap.*;

/**
 * This class is an embedded HTTP server which understands most of
 * the basic features. It uses FTAServlets to control its behaviour.
 */
public class FTAHttpServer extends Object {

    private static FTAService _svc = FTAService.Instance();

    /**
     * This is a standalone version of the server. It requires a
     * single argument which points to the absolute filename of
     * its httpd.cfg file (see jfta/http/http.cfg for a sample).
     * @param args argument vector (only args.get(0) used)
     */
    public static void main(String[] args) {
	if (args.length != 1)
	    usage();

	String conffile = args[0];
	Properties prop = new Properties();
	try {
	    prop.load(new FileInputStream(conffile));
	}
	catch (IOException e) {
	    System.out.println("FTAHttpServer: failed to load configfile " +
			       conffile);
	    System.exit(1);
	}

	try {
	    new FTAHttpServer(null,prop,new ThreadGroup("Http"));
	    System.out.println("FTAHttpServer: started");
	} catch(Exception e) {
	    System.out.println("FTAHttpServer: failed, exception="+e);
	}
    }

    private static void usage() {
	System.out.println("Usage: FTAHttpServer configfile");
	System.exit(1);
    }
    
    private String _baseurl;
    private Object _container;
    private Thread _handler;
    private Hashtable _servlets;
    private FTAServletI _error;
    private String _servername;
    private Properties _conf;
    private FTAHttpLogI _logger;
    private FTAServerLogServlet _servlog;
    private Hashtable _contents;
    private static int _debuglevel = 0;
    private ThreadGroup _mytg;

    /**
     * log a message
     */
    public void logMessage(int type,String msg) {
	_servlog.logMessage(type,msg);
    }

    /**
     * return the version of the server
     * @return string
     */
    protected String getVersion() {
	return "FTAHttpServer V1.0";
    }

    /**
     * obtain a property from the server's httpd.cfg
     * @param propname property key
     * @return proerty value
     */
    public String getProperty(String propname) {
	return _svc.expandEnv(_conf.getProperty(propname));
    }

    /**
     * obtain a property from the server's httpd.cfg
     * @param propname property key
     * @param def default value, if property not found
     * @return property value
     */
    public String getProperty(String name,String def) {
	return _svc.expandEnv(_conf.getProperty(name,def));
    }

    /**
     * constructor. This is the HTTP server module. You can instantiate
     * multiple servers, provided they have a different property set.
     * @param container the toplevel class where this server is embedded.
     * This is passed to the servlets through the request packet
     * (getServer().getcontainer()), to allow
     * servlets getting easy access to facilities outside the server.
     * This argument may be null.
     * @param conf the property set for the configuration. This is an
     * already parsed table from httpd.conf, for instance. It must
     * provide the following entries:
     * <ul><li>ServerPort (network port of the server)
     * <li>Servlets (list of toplevel servlets to be used)
     * <li>ErrorHandler (servlet to be used in case of error)
     * <li>ServerName (name of the server)
     * <li>ContentHandlers (list of content handlers for special MIME types)
     * <li>BaseURL (URL prefix to be prepended to the reply)
     * </ul>
     * It may also provide the following entries (but need not):
     * <li>DebugLevel (default 0, higher numbers will log more information)
     * <li>LogHandler (name of a class implementing the FTAHttpLogI
     * interface which produces output logs. If this entry is missing,
     * no logs are created).
     * </ul>
     */
    public FTAHttpServer(Object container,Properties conf,ThreadGroup tg) 
	throws Exception {
	_container = container;
	_conf = conf;
	_mytg = tg;
	int port = _svc.atoi(getProperty("ServerPort"),80);
	_servlets = FTAServlet.load(this,getProperty("Servlets"),
				    _servlets);
	_baseurl = getProperty("BaseURL");
	_error = (FTAServletI)FTAServlet.byName(getProperty("ErrorHandler"));
	_contents = FTAContentHandler.load(this,
					   getProperty("ContentHandlers"),
					   _contents);
	_servername = getProperty("ServerName");
	_debuglevel = _svc.atoi(getProperty("DebugLevel"),0);
	_logger = FTAHttpLogHandler.load(this,getProperty("LogHandler"));
	_servlog = new FTAServerLogServlet();

	// start a http thread
	_handler = new Handler(this,port);
	_handler.start();
    }

    /**
     * shutdown HTTP Server handling thread.
     */
    public void shutdown() {
	_handler.destroy();
    }

    private static void debug(int lvl,String msg) {
	if (_debuglevel > lvl) {
	    String h = _svc.getHostname();
	    System.out.println(h+": "+msg);
	}
    }

    private class Handler extends Thread {
	private ServerSocket _ss;
	private FTAHttpServer _server;

	public Handler(FTAHttpServer server,int port) {
	    super(_mytg,"HttpAccept");
	    _server = server;
	    try {
		_ss = new ServerSocket(port,100);
	    }
	    catch(IOException e) {
		e.printStackTrace();
		System.exit(1);
	    }
	    debug(0,"Accepting connections on port " + 
		  _ss.getLocalPort());
	}

	public void run() {
	    for (;;) {
		try {
		    Socket s = _ss.accept();
		    Thread dh = new Connection(_server,s);
		    dh.start();
		}
		catch(IOException e) {
		    e.printStackTrace();
		}
	    }
	}
    }

    /**
     * return the container where this server is embedded (or null)
     * @return container object
     */
    public Object getContainer() {
	return _container;
    }

    /********************************************************************/
    /* Connection Handler */
    /********************************************************************/

    private class Connection extends Thread {

	private Socket _conn;
	private int _contentlength;
	private FTAHttpServer _server;

	public Connection(FTAHttpServer server,Socket s) {
	    super(_mytg,"Http@"+s);
	    _server = server;
	    _conn = s;
	}

	/**
	 * Thread main call
	 */
	public void run() {
	    try {
		OutputStream os = _conn.getOutputStream();
		DataInputStream is = new DataInputStream
		    (new BufferedInputStream(_conn.getInputStream()));
		InetAddress hostaddr = _conn.getInetAddress();
		String host = hostaddr.getHostName();
		String authuser = null;
		
		// get the request
		String requ = is.readLine();
		debug(0,"Request is "+requ);
		
		// to upper case
		String upper = requ.toUpperCase();
		
		// accept GET and POST
		boolean postflag = upper.startsWith("POST");
		
		// parse MIME header
		Hashtable header = parse_mime_header(is,requ,upper);
		
		// retrieve the GET/POST token and the command
		StringTokenizer st = new StringTokenizer(requ);
		String tok = st.nextToken();
		String cmd = st.nextToken();
		
		header.put("X-Method",tok);
		header.put("X-Originating-Host",host);
		Hashtable args = null;
		
		// retrieve the arguments of a POST form (if applicable)
		if (postflag && _contentlength > 0)
		    args = parse_post_args(is,args,_contentlength);
		
		// retrieve also possible GET args
		int quest = cmd.indexOf('?');
		if (quest >= 0)
		    args = parse_get_args(cmd,args);
		String realpath = FTAServlet.splitHead(cmd,quest);
		
		debug(1,"GET/POST args="+args+
		      " realpath="+realpath);

		// perform the command
		FTAServletRequest req =
		    new FTAServletRequest(_server, _baseurl+cmd,
					  realpath, header, args, os);
		FTAServletReply reply = dispatchRequest(req,_servlets);
		
		// if the dispatcher didn't match anything,
		// and no default exists,
		// try as a last choice the server error handler
		if (reply == null) {
		    // custom error handler
		    FTAServletI sl =
			(FTAServletI)_servlets.get("error");
		    
		    // no, try system error handler
		    if (sl == null)
			sl = _error;

		    // no error handler? Too bad!
		    if (sl == null) {
			reply = 
			    FTAErrorReply.error500("No Error Handler");
		    } else {
			req.setErrorCode(404);
			reply = sl.reply(req);
		    }
		} 
		
		// send the output content back to the stream
		handle_content(req,reply);

		// log the request
		_logger.log(host,null,authuser,
			    requ,reply.getErrorCode(),
			    reply.getContentLength());


		// so close connection
		_conn.close();
	    }
	    catch (Exception e) {
		e.printStackTrace();
		System.out.println("IOException in HttpHandler "+e);
	    }
	}
	
	/**
	 * read the MIME header
	 */
	private Hashtable parse_mime_header(DataInputStream is,
					    String req,String upper) 
	    throws IOException {
	    if (upper.indexOf("HTTP/") >= 0) {
		Hashtable h = new Hashtable();
		while (true) {
		    // get line
		    String mimehdr = is.readLine();
		    
		    // exit if empty line
		    if (mimehdr.trim().equals("")) break;

		    // add to hash
		    int col = mimehdr.indexOf(':');
		    String key = FTAServlet.splitHead(mimehdr,col);
		    String val = col != -1 ? 
			(mimehdr.substring(col+1)).trim() :
			mimehdr;

		    h.put(key,val);
		    //System.out.println("mime: key="+key+" value="+val);
		    
		    // check content type for POST, only forms accepted
		    if (key.equals("Content-type") &&
			!val.equals("application/x-www-form-urlencoded"))
			System.out.println("Warning: "+mimehdr);
		
		    // check content length
		    else if (key.equals("Content-length")) {
			_contentlength = _svc.atoi(val,0);
		    }
		}
		return h;
	    } else {
		System.out.println("Warning: Unsupported outdated HTTP client: " + 
				   req);
	    }
	    return null;
	}
    }

    public
	FTAServletReply handle_content(FTAServletRequest req,
				      FTAServletReply reply) 
	throws IOException {
	String match = reply.getContentType();
	debug(3,"Content-type of reply=" + match);

	// get specific handler for this content
	FTAContentHandler ch = (FTAContentHandler)_contents.get(match);

	// not found, try a default handler
	if (ch == null) {
	    match = "default";
	    ch = (FTAContentHandler)_contents.get(match);
	}

	// no default handler? Signal internal error
	if (ch == null) {
	    debug(3,"Internal error: no default content handler");
	    reply.setErrorCode(500);
	} else {
	    debug(3,"Using content handler "+
		  ch.toString() +
		  " (matched by \"" +
		  match + "\")");
	    
	    ch.emit(req,reply);
	}
	return reply;
    }
    
    /**
     * dispatch next level of URL
     * @param req servlet request
     * @param servlets table of responsible servlets
     */
    public static 
	FTAServletReply dispatchRequest(FTAServletRequest req,
					Hashtable servlets) {
	String path = req.getTail();
	if (path.length()==0 ||
	    path.equals("/") ||
	    path.equals("/index.html"))
	    path = "/index.html";
	debug(2,"dispatchRequest: inpath="+path);
 
	// strip first part
	int r = path.indexOf('/',1);
	String head = FTAServlet.splitHead(path,r);
	String tail = FTAServlet.splitTail(path,r);

	// check if this is the last part of path, and whether
	// it contains a GET arg sequence
	r = head.indexOf('?');
	head = FTAServlet.splitHead(head,r);

	debug(2,"dispatchRequest: outhead=<" + head +
	      "> tail=<" + tail +
	      "> outpath=<" + path + ">");

	// select a matching servlet
	String match = head;
	FTAServletI sl = (FTAServletI)servlets.get(head);
	req.modifyHeadTail(head,tail);
	if (sl == null) {
	    // if this is a file (tail==null), try also
	    // extensionhandlers
	    int ex;
	    if ((ex=head.indexOf(".")) != -1) {
		match = head.substring(ex);
		sl = (FTAServletI)servlets.get(match);
	    }
	    
	    // attempt default handler, before falling to error handler
	    if (sl == null) {
		match = "default";
		sl = (FTAServletI)servlets.get(match);
	    }
	}

	// catch exceptions in servlets
	try {
	    if (sl != null) {
		debug(3,"Using servlet " + 
		      sl.toString() +
		      "(matched by \"" + 
		      match + "\")");
		return sl.reply(req);
	    } else {
		debug(3,"No matching servlet (404)");
		return FTAErrorReply.error404();
	    }
	}
	catch(Exception e) {
	    debug(3,"Internal error (500), exc="+e);
	    return new FTAErrorReply(500,e);
	}
    }

    private Hashtable split_args(StringTokenizer st, Hashtable args) {
	if (st.hasMoreTokens()) {
	    if (args==null)
		args = new Hashtable();
	    while (st.hasMoreTokens()) {
		String arg = st.nextToken();
		int eq = arg.indexOf('=');
		if (eq != -1) {
		    String name = FTAServlet.splitHead(arg,eq);
		    String value;
		    try {
			value = URLDecoder.decode(arg.substring(eq+1));
		    } catch(Exception e) {
			value = null;
		    }
		    if (value != null)
			args.put(name,value);
		}
	    }
	}
	return args;
    }

    /**
     * split argument list of a GET type CGI script (i.e. URL?arg1=val&arg2=val2...)
     * @param path
     * @return splitted argument list, null if no args found
     */
    private Hashtable parse_get_args(String url,Hashtable args) {
	int quest = url.indexOf('?');
	if (quest == -1)
	    return args;
	StringTokenizer st = new StringTokenizer(url.substring(quest+1),
						 "&");
	return split_args(st,args);
    }

    /**
     * retrieve arguments of a POST form
     */
    private Hashtable parse_post_args(DataInputStream is,Hashtable args,int clen) {
	byte[] buf = new byte[clen];
	try {
	    int nread = is.read(buf,0,clen);
	    String line = new String(buf);
	    if (line != null) {
		StringTokenizer st = new StringTokenizer(line,"&");
		return split_args(st,args);
	    }
	}
	catch (Exception e) {}
	return args;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
