// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAServlet.java,v 1.3 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.net.*;
import java.io.*;
import java.util.*;

import jfta.exception.*;

/**
 * The FTAServlet class implements specific page handlers. Each
 * servlet is called with an FTAServletRequest packet, and
 * has to return a reply as an FTAServletReply object.
 */
public class FTAServlet extends Object implements FTAServletI {

    /**
     * mandatory (empty) constructor
     */
    public FTAServlet() {}
    
    /**
     * initialize servlet with arguments
     * @param server backlink to server
     * @param argv argument vector
     * @exception FTAArgumentException raised by the derived class init
     * method, when some required argument is incorrect
     */
    public void init(FTAHttpServer server,Vector argv)
	throws FTAArgumentException {
	// default: do nothing
    }
    
    /**
     * return an output file to the calling server
     * @param req request data
     * @return reply
     */
    public FTAServletReply reply(FTAServletRequest req) {
	FTAHtmlFile f = new FTAHtmlFile("Internal Error in "+toString());
	f.text("Serveradmin must implement reply() method for Servlet " + 
	       toString());
	return f.finish();
    }
    
    /**
     * read servlets from file
     * @param filename file to read definitions from
     * @return a hashtable
     */
    protected static Hashtable load(FTAHttpServer server,
				    String filename,
				    Hashtable servlets)
	throws IOException {
	
	if (servlets == null)
	    servlets = new Hashtable();
	Properties p = new Properties();
	FileInputStream fis = new FileInputStream(filename);
	p.load(fis);
	
	Enumeration it = p.propertyNames();
	while (it.hasMoreElements()) {
	    String name = (String)it.nextElement();
	    String val = p.getProperty(name);
	    
	    StringTokenizer st = new StringTokenizer(val," \t\n");
	    String clsname = st.nextToken();
	    Vector args = null;
	    if (st.hasMoreTokens()) {
		args = new Vector();
		while (st.hasMoreTokens()) {
		    args.add(st.nextToken());
		}
	    }
	    
	    try {
		FTAServlet sl = byName(clsname);
		sl.init(server,args);
		servlets.put(name,sl);
		//System.out.println("Servlet " + clsname + " loaded");
	    }
	    catch(FTAArgumentException e) {
		throw new IOException(clsname + 
				      ".init() failed: " + e);
	    }
	    catch(Exception e) {
		throw new IOException("Creation of " + 
				      clsname + " failed: " + e);
	    }
	}
	return servlets;
    }
    
    /**
     * dispatch next level of URL
     * @param req servlet request
     * @param servlets table of responsible servlets
     */
    public static 
	FTAServletReply dispatchRequest(FTAServletRequest req,
					Hashtable servlets) {
	return FTAHttpServer.dispatchRequest(req,servlets);
    }

    /**
     * split the head off an URL part
     * @param str URL part
     * @param r index of '/' delimiter
     */
    public static String splitHead(String str,int r) {
	return r != -1 ? str.substring(0,r) : str;
    }

    /**
     * split the tail off an URL part
     * @param str URL part
     * @param r index of '/' delimiter
     */
    public static String splitTail(String str,int r) {
	return r != -1 ? str.substring(r) : null;
    }
    
    /**
     * Load a servlet by name
     * @param clsname java name of the servlet, e.g. jfta.demo.myservlet
     * @return created servlet instance
     * @exception Exception various exceptions while creating a new object 
     * may be raised
     */
    public static FTAServlet byName(String clsname)
	throws Exception {
	Class slclass = Class.forName(clsname);
	return (FTAServlet)slclass.newInstance();
    }
    
    /**
     * return the servlet classname
     * @return convert to string
     */
    public String toString() {
	return getClass().getName();
    }

    /**
     * return an absolute URL where the object is reachable by default.
     * Plain FTAServlets that may be instantiated everywhere, do not
     * have this method implemented, they return null.
     * @param n allows to return other URLs specified by this param
     * @return url
     */
    public String getURL(int n) {
	return null;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
