// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

// $Id: FTAServletReply.java,v 1.3 2001/06/21 14:57:24 veit Exp $

package jfta.http;

import java.net.*;
import java.util.*;
import java.io.*;
import jfta.base.*;

/**
 * This class describes the reply to be returned by a servlet, when
 * it is called for a request.
 */
public class FTAServletReply {
    protected String _contenttype;
    protected int _errorcode;
    protected Hashtable _headers;
    protected InputStream _stream;
    protected int _contentlength;

    /**
     * create a new reply of a given contenttype
     * @param contenttype the MIME content type
     */ 
    public FTAServletReply(String contenttype) {
	this(contenttype,0,null);
    }

    /**
     * create a new reply of a given contenttype, and contentlength
     * @param contenttype the MIME content type
     * @param contentlength the length of the returned content in bytes
     */ 
    public FTAServletReply(String contenttype,int contentlength) {
	this(contenttype,contentlength,null);
    }

    /**
     * create a new reply of a given contenttype, and contentlength,
     * and a stream to be read to produce the output.
     * @param contenttype the MIME content type
     * @param contentlength the length of the returned content in bytes
     * @param stream a stream which has to be read to return the
     * proper output. This may come from anywhere.
     */ 
    public FTAServletReply(String contenttype,int contentlength,
			   InputStream stream) {
	_contenttype = contenttype;
	_errorcode = 200;
	_contentlength = contentlength;
	_headers = null;
	_stream = stream;
    }

    /**
     * return the current content type
     * @return MIME content tpye string
     */ 
    public String getContentType() {
	return _contenttype;
    }

    /**
     * return the current reply status code
     * @return status code
     */ 
    public int getErrorCode() {
	return _errorcode;
    }

    /**
     * set the reply status code
     * @param new status code
     */
    public void setErrorCode(int errorcode) {
	_errorcode = errorcode;
    }

    /**
     * Finish creation of the output before returning the result
     * to the server. This is sometimes overridden by a derived class.
     * @return returns itself for simplicity in recursive calls
     */
    public FTAServletReply finish() { return this; }

    /**
     * return the stream to return the result data
     * @return a valid input stream
     */
    public InputStream getStream() {
	return _stream;
    }

    /**
     * return the content length of the result
     * @return an integer value
     */
    public int getContentLength() {
	return _contentlength;
    }

    /**
     * set the content length in bytes
     * @param length new content length
     */
    public void setContentLength(int length) {
	_contentlength = length;
    }

    /**
     * add a custom header line to be returned to the caller.
     * Some servlets need this, for instance to signal special
     * conditions to the browser (like cookies, authentication)
     * @param key the key part of the line
     * @param value the value part of the line
     */
    public void addCustomReply(String key,String value) {
	if (_headers == null)
	    _headers = new Hashtable();
	_headers.put(key,value);
    }

    /**
     * return the table of custom replies
     * @return a hashtable (or a null pointer)
     */
    public Hashtable getCustomReply() {
	return _headers;
    }

    /**
     * redirect reply to another URL
     * @param url URL to redirect reply
     */
    public void redirect(String url) {
	setErrorCode(301);
	addCustomReply("Location",url);
    }

    /**
     * provide a default reply for an object, to be used as a catchall
     * for objects that implement the FTAServletI interface.
     * @param obj 
     * @return returns a default reply that lists the class
     * and the name, if not null.
     */
    public static FTAServletReply defaultReply(Object obj, String name) {
	FTAHtmlFile f = new FTAHtmlFile("Information about this Object");
	f.text("This object is an instance of ");
	f.text(obj.getClass().getName());
	if (name != null) {
	    f.text(".<p>Its name is ");
	    f.text(name);
	}
	return f.finish();
    }

}
