// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTASysSub.java,v 1.24 2001/06/21 14:57:26 veit Exp $
 
package jfta.system;

import java.util.*;
import java.net.*;

import jfta.base.*;
import jfta.bootstrap.*;
import jfta.tags.*;
import jfta.tagboard.*;
import jfta.actors.*;
import jfta.ensemble.*;
import jfta.exception.*;
import jfta.http.*;

/**
 * This represents a remote host. It contains references to all
 * local objects (EM,ES,TM,TS) and the system name (for back connect).
 * It provides services to the actor.
 */
public class FTASysSub implements FTAServletI {

    /**
     * constants used for notifyControl
     */
    public static final int FTACTLINITIALIZING = 0;
    public static final int FTACTLSTART = 1;
    public static final int FTACTLSTOP = 2;
    public static final int FTACTLSHUTDOWN = 3;

    public static final String[] SysStates = {
	"INITIALIZING",
	"RUNNING",
	"STOPPED",
	"TERMINATED",
    };

    private static FTAService _svc = FTAService.Instance();

    private FTADaemon _mydaemon;
    private String _mysysname;
    private String _myhostname;
    private String _mymainhost;
    private String _myurl;
    private int _myobjnum;
    private int _myobjcnt;
    private int _sysstate;
    private FTAListenerAdapter _listener;
    private FTASysTagboard _sysinfo;
    private FTACommManager _comm;
    private Hashtable _localm;
    private Hashtable _localsByName;
    private FTAIHash _localsByNum;
    private Hashtable _idcache;
    private boolean _isfirst;
    private boolean _initializing;
    private StringBuffer _defaultlog;
    private int _defaultlogcount;
    private FTASemaphore _finishsem;
    private ThreadGroup _systg;
    private SysEvalThread _sysevalthread;
    private FTAEvalable _frontqueue;
    private FTAEvalable _backqueue;
    private long _evalcnt;

    /**
     * ctor when started as a system start office
     */
    public FTASysSub(FTADaemon daemon,String mainhost, String sysname,
		     String acname,String[] arglist) throws Exception {
	this(daemon,mainhost,sysname,0,0,acname,arglist);
    }

    /**
     * ctor when started as a secondary office
     */
    public FTASysSub(FTADaemon daemon, String mainhost, String sysname,
		     int backport, int hostnum,
		     String acname,String[] arglist)
	throws Exception {

	// initialize certain vars
	_initializing = _isfirst = (acname!=null);
	_sysstate = FTACTLINITIALIZING;

	_mydaemon = daemon;
	_mymainhost = mainhost;
	_mysysname = sysname;
	_myhostname = _svc.getFQDNname();
	_systg = new ThreadGroup(sysname);

	_myobjnum = hostnum;
	_myobjcnt = 0;

	if (_isfirst) {
	    _defaultlog = new StringBuffer();
	    _defaultlogcount = 0;
	    _myurl = FTADaemon.buildURL(_myhostname,"/fta/"+sysname);

	    // initialize the startup semaphore
	    _finishsem = new FTASemaphore();
	}

	// manage complete communication for this host
	_comm = new FTATCPCommManager(this,mainhost,
				      backport,hostnum);
	// create an infoboard and add itself to it
	String url = FTADaemon.buildURL(_myhostname,
					"/fta/"+_mysysname+
					"/$SYSINFO$/");
	_sysinfo = new FTASysTagboard(url);
	FTASysTag tag = new FTASysTag(32767,FTASysTag.TB,
				      _myhostname,"$SYSINFO$");
	_sysinfo.putTag1(tag);

	// local stores
	_localm = new Hashtable();
	_localsByName = new Hashtable();
	_localsByNum = new FTAIHash();

	// install message handler for broadcast
	_listener = new SSListener();

	// start the syseval thread
	_sysevalthread = new SysEvalThread();
	_sysevalthread.start();

	// send a broadcast message that I am operable
	_comm.broadcastMsg
	    (FTACreateHostDoneMsg.get(hostnum,
				      _myhostname,
				      _comm.getServerPort()));

	// giveup this timeslice
	// to allow receiver threads to run
	Thread.yield();

	if (_initializing) {
	    // now run the init actor
	    Class initclass = Class.forName(acname);
	    Object initobj = initclass.newInstance();
	    
	    if (initobj instanceof FTAInitActor) {
		// call the base initializer
		FTAInitActor i = (FTAInitActor)initobj;
		logMessage(0,"Calling initactor");
		i.baseclassinit(this);
		i.init(arglist);
		
		// call the real init code
		i.eval();
		logMessage(3,"Finished initactor");
		
		// unregister init callbacks
		_listener.initdone(_comm);
		
		// mark end of initialization phase
		_initializing = false;

		// notify also system log
		_mydaemon.logMessage(0,_mysysname+": init done");

		// broadcast all host information
		_comm.distributeHostInfo();

	    } else
		throw new FTAInitException("Invalid actor type: " + 
					   acname);
	}
    }

    /************************************************************/
    /* This manages the scheduler list */
    /************************************************************/

    /**
     * why do I implement yet another boubly linked list here?
     * Because of performance!
     */
    private class FTAEvalable {
	FTAEvalable evnext,evprev;
	boolean evkill;
	FTAEvalableI evelem;
	
	FTAEvalable(FTAEvalableI elem,FTAEvalable next,FTAEvalable prev) {
	    evelem = elem;
	    evnext = next;
	    evprev = prev;
	    evkill = false;
	}
    }

    private class SysEvalThread extends Thread {
	private FTAEvalable evalhead,evaltail;

	public SysEvalThread() {
	    super(_systg,"Eval");
	    evalhead = evaltail = null;
	}
	
	public void run() {
	    while (_sysstate != FTACTLSHUTDOWN) {
		// schedule all evalables
		scheduleEvalables();
		// process queues
		processEvalQueues();
		Thread.yield();
		_evalcnt++;
	    }
	    //System.out.println("sysevalthread terminated");
	}

	/**
	 * add an FTAEvalableI to the schedule
	 * this is the actual management of the list
	 */
	private void addFront(FTAEvalable e) {
	    /**
	     *   evaltail------------------------+
	     *             +---+        +---+    +--->+---+
	     *   evalhead->| e |->next->|   |       ->|   |
	     *       null<-|   |<-prev<-|   |       <-|   |
	     *             +---+        +---+         +---+
	     */
	    //System.out.println("addFront: "+e);
	    e.evnext = evalhead;
	    e.evprev = null;
	    if (evalhead == null)
		evaltail = e;
	    else
		evalhead.evprev = e;
	    evalhead = e;
	}
	
	/**
	 * add an FTAEvalable to the schedule
	 * this is the actual management of the list
	 */
	public void addBack(FTAEvalable e) {
	    /**
	     *   evaltail-------------------------+
	     *             +---+        +---+     +--->
	     *   evalhead->| e |->next->|   |
	     *       null<-|   |<-prev<-|   |
	     *             +---+        +---+
	     */
	    //System.out.println("addBack: "+e);	
	    if (evaltail == null)
		addFront(e);
	    else {
		e.evnext = null;
		evaltail.evnext = e;
		e.evprev = evaltail;
		evaltail = e;
	    }
	}
	
	/**
	 * remove an FTAEvalable to the schedule
	 * @return next evalable
	 */
	private FTAEvalable removeEvalable(FTAEvalable e) {
	    e.evelem = null;
	    if (e==evaltail) {
		/*
		 * remove last element:
		 * evaltail-+
		 * +---+    +-->+---+    
		 * |   |->next->| ev|->next->null
		 * |   |<-prev<-|   |
		 * +---+        +---+
		 */
		evaltail = e.evprev;
		if (evaltail == null) // no predecessor: was single element
		    evalhead = null;
		else
		    evaltail.evnext = null;
		return null;
	    } else if (e==evalhead) {
		/*
		 * can't be last element (already matched above)
		 * evalhead-+
		 *          +-->+---+        +---+
		 *              | ev|->next->|   |
		 *  null<-prev<-|   |<-prev<-|   |
		 *              +---+        +---+
		 */
		evalhead = e.evnext;
		if (evalhead != null)
		    evalhead.evprev = null;
		return evalhead;
	    } else {
		/*
		 * is somewhere in the middle (at least three elements)
		 *  +---+        +---+        +---+
		 *  |   |->next->| e |->next->|   |
		 *  |   |<-prev<-|   |<-prev<-|   |
		 *  +---+        +---+        +---+
		 */
		e.evprev.evnext = e.evnext;
		if (e.evnext != null) 
		    e.evnext.evprev = e.evprev;
		return e.evnext;
	    }
	}

	/**
	 * execute the evalables
	 */
	private void scheduleEvalables() {
	    FTAEvalable cur = evalhead;
	    while (cur != null) {
		if (cur.evkill) {
		    /* discard this object */
		    cur = removeEvalable(cur);
		} else {
		    if (cur.evelem.isEval()) {
			try {
			    /* run it */
			    if (cur.evelem.eval() == FTAEvalableI.TERMINATE)
				cur.evkill = true; /* mark fo kill */
			}
			catch (FTAEvalException e) {
			    /* some exception occurred */
			    logMessage(FTAActor.LOG_ERROR,e.getMessage());
			    cur.evkill = true; /* mark for kill */
			}
		    }
		    cur = cur.evnext;
		}
	    }
	}

	private void processEvalQueues() {
	    FTAEvalable e;
	    while ((e=dequeueFront()) != null)
		addFront(e);
	    while ((e=dequeueBack()) != null)
		addBack(e);
	}

	public FTAEvalable getList() {
	    return evalhead;
	}
    }

    // the following methods are present to synchronize
    // adding to the sysevalthread Evalable list with
    // asynchronously incoming new requests
    
    private synchronized void enqueueFront(FTAEvalableI e) {
	//System.out.println("enqueuefront: "+e);
	_frontqueue = new FTAEvalable(e,_frontqueue,null);
    }

    private synchronized void enqueueBack(FTAEvalableI e) {
	//System.out.println("enqueueback: "+e);
	_backqueue = new FTAEvalable(e,_backqueue,null);
    }

    private synchronized FTAEvalable dequeueFront() {
	FTAEvalable e = _frontqueue;
	if (e != null) {
	    //System.out.println("dequeuefront: "+e);
	    _frontqueue = e.evnext;
	    e.evnext = e.evprev = null;
	}
	return e;
    }

    private synchronized FTAEvalable dequeueBack() {
	FTAEvalable e = _backqueue;
	if (e != null) {
	    //System.out.println("dequeueback: "+e);
	    _backqueue = e.evnext;
	    e.evnext = e.evprev = null;
	}
	return e;
    }

    /**
     * list actors that belong to an ensemble
     */
    public synchronized void listActors(FTAHtmlFile f,FTAEnsembleSub es) {
	String myurl = es.getURL(0);
	FTAEvalable e = _sysevalthread.getList();
	while (e != null) {
	    FTAEvalableI evi = e.evelem;
	    if (evi instanceof FTAActor) {
		FTAActor ac = (FTAActor)evi;
		if (ac.belongs(es)) {
		    String acname = ac.getName();
		    f.item();
		    f.anchor(myurl+acname+"/",acname);
		}
	    }
	    e = e.evnext;
	}
    }

    /**
     * locate an actor
     */
    public synchronized FTAActor findActor(String acname) {
	FTAEvalable e = _sysevalthread.getList();
	while (e != null) {
	    FTAEvalableI evi = e.evelem;
	    if (acname.equals(evi.toString()) && evi instanceof FTAActor)
		return (FTAActor)evi;
	    e = e.evnext;
	}
	return null;
    }

    /**
     * @roseuid 38B3C2B5F95C
     */
    public void controlSystem(int function, int code) {
	switch (function) {
	case FTACTLINITIALIZING:
	    return;
	default:
	    _sysstate = function;
	    if (_isfirst) {
		_comm.broadcastMsg(FTAControlMsg.get(function,code));
	    }
	}
    }

    /**
     * get the status
     * @return system status 
     */
    public int getSystemStatus() {
	if (_sysstate != FTACTLSHUTDOWN) {
	    long time = System.currentTimeMillis();
	    //_comm.broadcastMsg(FTAIdReqMsg.get(time));
	}
	return _sysstate;
    }
    
    /**
     * get the threadgroup of this FTA system
     */
    public ThreadGroup getSysThreadGroup() {
	return _systg;
    }

    /**
     * called when the object will be destroyed
     */
    protected void deinstall() {

	// dissociate from message dispatcher
	_listener.uninstall(_comm);
	
	/* deregister at daemon */
	_mydaemon.removeSystem(_mysysname);

	/* close the low level connection */
	_comm.uninstall();
    }

    /**
     */
    public final String toString() {
	return "SS"+_myobjnum;
    }

    /**
     */
    public final int getObjnum() {
	return _myobjnum;
    }

    /**
     */
    public final String getName() {
	return _mysysname;
    }

    /**
     * ask for the current list of available hosts.
     * Note: this is the list of <b>known</b> hosts (i.e. those
     * where an FTADaemon replies to the query request), not the
     * list of hosts that participate in this FTASystem.
     * @return a list of hostnames
     */
    public String[] getHosts() {
	Hashtable h = _mydaemon.getDaemons();
	String[] r = new String[h.size()];
	Enumeration it = h.keys();
	int i = 0;
	while (it.hasMoreElements()) {
	    r[i++] = (String)it.nextElement();
	}
	return r;
    }

    /**
     * get the name of an object
     */
    public String getNameById(int objnum) {
	return _sysinfo.getTagNameById(objnum);
    }

    /**
     * log a message
     */
    public final void logError(String msg) {
	logMessage(1,msg);
    }

    /**
     * log a message
     */
    public final void logWarning(String msg) {
	logMessage(2,msg);
    }

    /**
     * log a message
     */
    public final void logSuccess(String msg) {
	logMessage(3,msg);
    }

    /**
     * log a message
     */
    public final void logMessage(String msg) {
	logMessage(0,msg);
    }

    /**
     * log a message
     */
    public void logMessage(int type,String msg) {
	if (msg.length() > 100) msg = msg.substring(0,100);
	if (_isfirst) {
	    _defaultlogcount++;
	    if ((_defaultlogcount%1000)==0) {
		_defaultlog = new StringBuffer(10000);
		FTAServerLogServlet.logMessage(_defaultlog,2,
					       "["+_defaultlogcount+" messages lost]");
	    }
	    FTAServerLogServlet.logMessage(_defaultlog,type,msg);
	} else {
	    _comm.sendMsg(0,FTALogMsg.get(type,msg));
	}
    }

    /**
     * get the local communication manager
     */
    public final FTACommManager getCommManager() {
	return _comm;
    }

    /**
     * Register a new host for use by the current FTA system.
     * @param hostname  the fully qualified domain name of a host
     * @exception FTACreationException if the creation failed
     */
    public final void addHost(String hostname) 
	throws FTACreationException {

	// call the remote FTADaemon to instantiate a new FTASysSub.
	if (_isfirst) {
	    // ignore a request to create another one on the mainhost
	    if (_mymainhost.equals(hostname))
		return;

	    // get a new id
	    _myobjcnt++;

	    try {
		// contact FTADaemon
		URL daemon = _svc.getDaemonAt(hostname,
					      "/cgi-bin/newoffice.ftad");
		// send a command to the remote FTADaemon
		String query = "sysname=" + URLEncoder.encode(_mysysname);
		query += "&main=" + URLEncoder.encode(_svc.getFQDNname());
		query += "&bport=" + _comm.getServerPort();
		query += "&obj=" + _myobjcnt;

		_svc.sendHTMLCommandDone(daemon,query);
	    }
	    catch(Exception e) {
		e.printStackTrace();
		throw new FTACreationException("addHost: exc="+e);
	    }
	} else {
	    // send request to first syssub
	    _comm.sendMsg(0,FTACreateHostMsg.get(hostname));
	}

	// wait until finished (init only)
	waitDone();
    }
  
    /**
     * Add a new tagboard to the current FTA system.
     * @param tbname  logical name of the tagboard. Must be unique in
     *   this FTA system.
     * @param flags type of tagboard
     * @exception FTACreationException  the tagboard could not be
     *   created.
     */
    public final void addTagboard(String tbname, int flags)
	throws FTACreationException {

	if (_sysinfo.hasTBByName(tbname))
	    throw new FTACreationException("Object " + 
					   tbname+" already exists");
	else {
	    _comm.sendMsg(0,FTACreateTBMsg.get(_mymainhost,tbname));

	    // wait until object created
	    waitDone();
	}
    }
  
    /**
     * Add a new ensemble to the FTA system.
     * @param enname  logical name of the ensemble. Must be unique
     *   in this FTA system.
     * @param flags creation type
     * @exception FTACreationException  the ensemble could not be
     *   created.
     */
    public final void addEnsemble(String enname, int flags)
	throws FTACreationException {

	if (_sysinfo.hasENByName(enname))
	    throw new FTACreationException("Object "+ 
					   enname+" already exists");
	else {
	    _comm.sendMsg(0,FTACreateENMsg.get(_mymainhost,enname));

	    // wait until object created
	    waitDone();
	}
    }

    /**
     * handle put tag on behalf of the actor
     */
    public final void putTag(String tbname, String tgtype, String[] args)
	throws FTAUnknownObjectException,FTATagException,FTAArgumentException {

	Class tagclass;
	Object tagobj;
      
	try {
	    /* create an actor instance */
	    tagclass  = Class.forName(tgtype);
	    tagobj = tagclass.newInstance();
	}
	catch (Exception e) {
	    throw new FTATagException(e.toString());
	}

	/* could have been created */
	FTATag tag;
	if (tagobj instanceof FTATag) {
	    /* call baseclass init for tag */
	    tag = (FTATag)tagobj;
	
	    /* regular user init */
	    tag.init(args);
	} else
	    throw new FTATagException("Can't create tag "+tgtype);

	// submit this tag
	putTag(tbname,tag);
    }
  
    /**
     * handle put tag on behalf of the actor
     */
    public final void putTag(String tbname, FTATag tag)
	throws FTAUnknownObjectException,FTATagException {

	int tbnum = _sysinfo.getTBByName(tbname);
	_comm.broadcastMsg(FTAPutTagMsg.get(tbnum,tag));
    }

    /**
     * handle ET sync on behalf of the actor
     */
    public final void addETSync(String enname, String tbname)
	throws FTACreationException,FTAUnknownObjectException {

	int ennum = _sysinfo.getENByName(enname);
	int tbnum = _sysinfo.getTBByName(tbname);
	_comm.sendMsg(0,FTAAddETSyncMsg.get(ennum,tbnum));
    }
  
    /**
     * local addition of an actor
     */
    public final void localAddActor(FTAEnsembleSub es,FTAActor ac) {
	ac.baseclassinit(es);
	ac.postInit();
	enqueueFront(ac);
    }

    public final FTAActor createActor(String actype,String acname,String[] args)
	throws FTACreationException,FTAArgumentException {

	Class actorclass;
	Object actorobj;
	FTAActor actor;
      
	try {
	    /* create an actor instance */
	    actorclass  = Class.forName(actype);
	    actorobj = actorclass.newInstance();
	}
	catch (Exception e) {
	    throw new FTACreationException(e.toString());
	}

	if (actorobj instanceof FTAActor) {
	    /* call baseclass init for actor */
	    actor = (FTAActor)actorobj;
	    actor.baseclassinit(acname);
	
	    /* and regular user init */
	    actor.init(args);
	} else
	    throw new FTACreationException("Can't create actor "+actype);

	return actor;
    }

    /**
     */
    private final void addActor(String hostname, 
				String enname,
				FTAActor actor) 
	throws FTACreationException,FTAUnknownObjectException {

	int ennum = _sysinfo.getENByName(enname);
	if (hostname == null)
	    hostname = _myhostname;
	int hostnum = _sysinfo.getHostByName(hostname);
	if (FTADebug.AC)
	    FTADebug.log(_myobjnum,0,
			 "addactor: on " + hostnum +
			 " at " + ennum +
			 " ac=" + actor.getName());
	_comm.broadcastMsg(FTAAddActorMsg.get(hostnum,ennum,actor));
    }

    /**
     * handle add actor on behalf of an actor
     */
    public final void addActor(String hostname, String enname,
			       String acname,FTAActor actor)
	throws FTACreationException,FTAUnknownObjectException {

	actor.baseclassinit(acname);
	addActor(hostname,enname,actor);
    }

    /**
     */
    public final void addActor(String hostname, String enname,
			       String acname, String actype, String[] args)
	throws FTACreationException,FTAUnknownObjectException,FTAArgumentException {
	/* could have been created */
	FTAActor actor = createActor(actype,acname,args);

	addActor(hostname,enname,actor);
    }

    /**
     * Remove a host from the current FTA system. All objects 
     * that exist on this host will become unavailable afterwards.
     * This function will remove the system office reference for
     * this host, not the <code>FTADaemon</code>.
     * @param hostname fully qualified name of the host to be removed.
     * @exception FTARemovalException the host couldn't be removed
     * @exception FTAUnknownObjectException the host does not exist
     */
    public final void removeHost(String hostname)
	throws FTARemovalException,FTAUnknownObjectException {
	int hostnum = _sysinfo.getHostByName(hostname);
	// broadcast this to everybody
	_comm.broadcastMsg(FTARemHostMsg.get(hostnum));
    }

    /**
     * propagate a fliporder to the main EN office
     */
    public void flipOrder(int ennum,String tbname)
	throws FTAUnknownObjectException {
	int tbnum = _sysinfo.getTBByName(tbname);
	_comm.sendMsg(0,FTAFlipOrderReqMsg.get(ennum,tbnum));
    }
    /**
     * Remove a tagboard. The removal will take place
     * at the end of the ongoing flip.
     * @param tbname logical name of the tagboard
     * @exception FTARemovalException the tagboard couldn't be removed
     * @exception FTAUnknownObjectException the tagboard does not exist
     */
    public final void removeTagboard(String tbname)
	throws FTARemovalException,FTAUnknownObjectException {
	int tbnum = _sysinfo.getTBByName(tbname);
	_comm.broadcastMsg(FTARemTBMsg.get(tbnum));
    }
  
    /**
     * Remove an ensemble. The removal will take place
     * at the end of the ongoing tick.
     * @param enname logical name of the ensemble
     * @exception FTARemovalException the ensemble couldn't be removed
     * @exception FTAUnknownObjectException the ensemble does not exist
     * @roseuid 38B3C2E9E262
     */
    public final void removeEnsemble(String enname)
	throws FTARemovalException,FTAUnknownObjectException {
	int ennum = _sysinfo.getENByName(enname);
	_comm.broadcastMsg(FTARemENMsg.get(ennum));
    }
  
    /**
     * handle remove ET sync on behalf of the actor
     */
    public final void removeETSync(String enname, String tbname)
	throws FTARemovalException,FTAUnknownObjectException { 
	int ennum = _sysinfo.getENByName(enname);
	int tbnum = _sysinfo.getTBByName(tbname);
	_comm.sendMsg(0,FTARemETSyncMsg.get(ennum,tbnum));
    }

    private final int addTagboardMain(String tbname)
	throws FTACreationException {

	// get a new object number
	_myobjcnt++;

	try {
	    FTAEvalableI tm = new FTATagboardMain(tbname,this,_myobjcnt,
						  _sysstate==FTACTLSTART);
	    _localm.put(tbname,tm);
	    enqueueBack(tm);
	    return _myobjcnt;
	}
	catch(Exception e) {
	    throw new FTACreationException("local creation failed: ",tbname);
	}
    }

    /**
     * @roseuid 3871FEED3248
     */
    private final int addEnsembleMain(String enname)
	throws FTACreationException {
      
	// get a new obj number
	_myobjcnt++;

	try {
	    FTAEvalableI em = new FTAEnsembleMain(enname,this,_myobjcnt,
						 _sysstate==FTACTLSTART);
	    _localm.put(enname,em);
	    enqueueBack(em);
	    return _myobjcnt;
	}
	catch(Exception e) {
	    throw new FTACreationException("local creation failed: ",
					   enname);
	}
    }
  
    /**
     * get a local tagboard sub by id
     */
    private FTATagboardSub getTagboardById(int tbnum)
	throws FTAUnknownObjectException {
	Object obj = _localsByNum.get(tbnum);
	if (obj == null) {
	    FTADebug.log(_myobjnum,0,
			 "must create tagboard on the fly: "+tbnum);
	    // create such an object on the fly
	    return createTagboard(tbnum);
	}
	return (FTATagboardSub)obj;
    }
    
    /**
     * create a tagboard sub on the fly
     */
    private FTATagboardSub createTagboard(int tbnum) {
	String tbname = _sysinfo.getTagNameById(tbnum);
	FTATagboardSub ts = new FTATagboardSub(this,tbname,tbnum);
	_localsByName.put(tbname,ts);
	_localsByNum.put(tbnum,ts);
	enqueueBack(ts);
	if (FTADebug.CR)
	    FTADebug.log(_myobjnum,0,"Created new TBSub "+
			 tbname+"@"+_myhostname);
	return ts;
    }
    
    /**
     * get a local tagboard sub
     */
    private FTATagboardSub getTagboardByName(String name) 
	throws FTAUnknownObjectException {
	Object obj = _localsByName.get(name);
	if (obj == null) {
	    /**
	     * no such object found, is there at least a main
	     * object of this name?
	     */
	    int tbnum = _sysinfo.getTBByName(name);
	    return createTagboard(tbnum);
	} else if (obj instanceof FTATagboardSub)
	    return (FTATagboardSub)obj;
	throw new FTAUnknownObjectException("Invalid object type");
    }
    
    /**
     * get a local ensemble sub by id
     */
    private FTAEnsembleSub getEnsembleById(int ennum)
	throws FTAUnknownObjectException {
	Object obj = _localsByNum.get(ennum);
	if (obj == null) {
	    //FTADebug.log(_myobjnum,0,
	    //  "must create ensemble on the fly: "+id);
	    // create such an object on the fly
	    return createEnsemble(ennum);
	}
	return (FTAEnsembleSub)obj;
    }
    
    /**
     * create an ensemble sub on the fly
     */
    private FTAEnsembleSub createEnsemble(int ennum) {
	// create a ensemble office on the fly
	String enname = _sysinfo.getTagNameById(ennum);
	FTAEnsembleSub es = new FTAEnsembleSub(this,enname,ennum,true);
	_localsByName.put(enname,es);
	_localsByNum.put(ennum,es);
	enqueueBack(es);

	if (FTADebug.CR)
	    FTADebug.log(_myobjnum,0,
			 "Created new ENSub "+
			 enname+ " at " +_myhostname);
	return es;
    }

    /**
     * remove ensemble sub on its own request
     */
    public void removeEnsembleSub(int ennum, String logname) {
	FTAEnsembleSub es = (FTAEnsembleSub)_localsByName.remove(logname);
	_localsByNum.remove(ennum);
    }

    /**
     * get a local ensemble sub
     */
    private FTAEnsembleSub getEnsembleByName(String name) 
	throws FTAUnknownObjectException {
	Object obj = _localsByName.get(name);
	if (obj == null) {
	    /**
	     * no such object found, is there at least a main
	     * object of this name?
	     */
	    int ennum = _sysinfo.getENByName(name);
	    return createEnsemble(ennum);
	} else if (obj instanceof FTAEnsembleSub)
	    return (FTAEnsembleSub)obj;
	throw new FTAUnknownObjectException("Invalid object type");
    }
    
    /**
     * @roseuid 38B3C2B0CC4C
     */  public long getTagboardTime(String name, int format)
	 throws FTAUnknownObjectException {
	 
	 FTATagboardSub ts = getTagboardByName(name);
	 return ts.getTagboardTime(format);
     }
    
    /**
     * @roseuid 38B3C2B0CC4F
     */
    public final long getEnsembleTime(String name, int format)
	throws FTAUnknownObjectException {
	
	FTAEnsembleSub es = getEnsembleByName(name);
	return es.getEnsembleTime(format);
    }
  
    /**
     * get the number of tags on a tagboard that match a certain pattern
     */
    public final int getTagCount(String tbname,String tagname)
	throws FTAUnknownObjectException {
      
	FTATagboardSub ts = getTagboardByName(tbname);
	return ts.getTagCount(tagname);
    }
  
    public FTATagIterator getTagIterator(String tbname, String tagname)
	throws FTAUnknownObjectException {
      
	FTATagboardSub ts = getTagboardByName(tbname);
	return ts.getTagIterator(tagname);
    }
  
    /**
     */
    public String[] getTagTypes(String tbname)
	throws FTAUnknownObjectException {
      
	FTATagboardSub ts = getTagboardByName(tbname);
	return ts.getTagTypes();
    }
  
    /**
     * wait until init operation is done
     */
    protected void waitDone() {
	if (_initializing)
	    _finishsem.P();
    }

    /**
     * signal an incoming reply
     */
    protected void signalDone() {
	if (_initializing) {
	    _finishsem.V(1);
	}
    }

    /************************************************************/
    /* this adapter manages registered messages */
    /************************************************************/

    private class SSListener extends FTAListenerAdapter {
	public SSListener() {
	    super();

	    _comm.addListener(FTAMsg.CONTROL,this);
	    _comm.addListener(FTAMsg.ADDACTOR,this,_myobjnum);
	    _comm.addListener(FTAMsg.CREATEHOST,this);
	    _comm.addListener(FTAMsg.CREATETB,this);
	    _comm.addListener(FTAMsg.CREATEEN,this);
	    _comm.addListener(FTAMsg.REMOVEHOST,this);
	    _comm.addListener(FTAMsg.REMOVEEN,this);
	    _comm.addListener(FTAMsg.REMOVETB,this);
	    _comm.addListener(FTAMsg.CREATETBDONE,this);
	    _comm.addListener(FTAMsg.CREATEENDONE,this);
	    _comm.addListener(FTAMsg.CREATEHOSTDONE,this);
	    if (_isfirst) {
		_comm.addListener(FTAMsg.LOG_MESSAGE,this);
	    }
	}

	public void uninstall(FTACommManager comm) {
	    comm.removeListener(FTAMsg.CONTROL,this);
	    comm.removeListener(FTAMsg.ADDACTOR,this);
	    comm.removeListener(FTAMsg.CREATEHOST,this);
	    comm.removeListener(FTAMsg.CREATETB,this);
	    comm.removeListener(FTAMsg.CREATEEN,this);
	    comm.removeListener(FTAMsg.REMOVEHOST,this);
	    comm.removeListener(FTAMsg.REMOVEEN,this);
	    comm.removeListener(FTAMsg.REMOVETB,this);
	    comm.removeListener(FTAMsg.CREATEHOSTDONE,this);
	    comm.removeListener(FTAMsg.CREATETBDONE,this);
	    comm.removeListener(FTAMsg.CREATEENDONE,this);
	    comm.removeListener(FTAMsg.LOG_MESSAGE,this);
	}

	public synchronized void notifyLogMessage(int type,String msg) {
	    logMessage(type,msg);
	}

	// issued by some SysSub to add new host
	public synchronized void notifyCreateHost(String hostname) {
	    // received from SysSub: request to add a host
	    try {
		addHost(hostname);
	    }
	    catch(FTACreationException e) {
		// ignore
	    }
	}

	// issued by SysSub that this new host is operable
	public synchronized void notifyCreateHostDone(int hostnum,
					 String hostname,
					 int port) {
	    if (FTADebug.CR)
		FTADebug.log(_myobjnum,0,
			     "createhostdone: "+hostname);
	    _sysinfo.putTag1(new FTASysTag(hostnum,FTASysTag.HOST,
					   hostname,hostname));
	    // only signal done, if this does not come from first host
	    if (hostnum != 0)
		signalDone();
	}

	// issued by SysMain to control operation
	public synchronized void notifyControl(int function, int code) {
	    if (FTADebug.CT)
		FTADebug.log(_myobjnum,0,"control: "+function);

	    _sysstate = function;

	    if (function >= 0 && function <= FTACTLSHUTDOWN) {
		if (_isfirst)
		    logSuccess("Change control: new state "+
			       SysStates[function]);
		if (function==FTACTLSHUTDOWN)
		    deinstall();
	    } else
		logError("Invalid control function " + 
			 function + ", code=" + code);
	}
    
	// issued by SysSub to create a new TB
	public synchronized void notifyCreateTB(String hostname,String logname) {
	    if (_isfirst) {
		try {
		    // create a TBMain here
		    int tbnum = addTagboardMain(logname);
		    _sysinfo.putTag1(new FTASysTag(tbnum,FTASysTag.TB,
						   hostname,logname));
		}
		catch(FTACreationException e) {
		    FTADebug.error(_myobjnum,0,
				   "Creation of TB " +
				   logname + " at " + 
				   hostname + " failed");
		}
	    }
	}

	// issued by TBMain when TBMain is operable
	public synchronized void notifyCreateTBDone(int tbnum,String hostname,
				       String logname,int myport) {
	    if (FTADebug.CR)
		FTADebug.log(_myobjnum,0, 
			     "createtbdone: "+logname+
			     " id="+tbnum);
	    _sysinfo.putTag1(new FTASysTag(tbnum,FTASysTag.TB,
					   hostname,logname));
	    signalDone();
	}

	// issued by SysSub to create a new EN
	public synchronized void notifyCreateEN(String hostname,String logname) {
	    if (_isfirst) {
		try {
		    // create a TBMain here
		    int ennum = addEnsembleMain(logname);
		    _sysinfo.putTag1(new FTASysTag(ennum,FTASysTag.EN,
						  hostname,logname));
		}
		catch(FTACreationException e) {
		    FTADebug.error(_myobjnum,0,
				   "Creation of EN " +
				   logname + " at " + 
				   hostname + " failed");
		}
	    }
	}

	// issued by ENMain when ENMain is operable
	public synchronized void notifyCreateENDone(int ennum,String hostname,
				       String logname,int myport) {
	    if (FTADebug.CR)
		FTADebug.log(_myobjnum,0,
			     "createendone: "+logname+
			     " id="+ennum);
	    _sysinfo.putTag1(new FTASysTag(ennum,FTASysTag.EN,
					   hostname,logname));
	    signalDone();
	}

	// issued by SysSub to remove host (all Mains get this)
	public synchronized void notifyRemoveHost(int hostnum) {
	    if (FTADebug.CR)
		FTADebug.log(_myobjnum,0,"removehost "+hostnum);
	    _sysinfo.removeTag(FTASysTag.HOST,hostnum);
	}
	
	// issued by SysSub to remove TB (all SysSubs get this)
	public synchronized void notifyRemoveTB(int tbnum) {
	    if (FTADebug.CR)
		FTADebug.log(_myobjnum,0,"removetb: "+tbnum);
	    _sysinfo.removeTag(FTASysTag.TB,tbnum);
	}
	
	// issued by SysSub to remove EN (all Syssubs get this)
	public synchronized void notifyRemoveEN(int ennum) {
	    if (FTADebug.CR)
		FTADebug.log(_myobjnum,0,"removeen: "+ennum);
	    _sysinfo.removeTag(FTASysTag.EN,ennum);
	}
	
	public synchronized void notifyAddActor(int ennum,FTAActor actor) {
	    if (FTADebug.AC)
		FTADebug.log(_myobjnum,0,
			     "notify: addactor: "+
			     actor.getType()+" ennum=" + ennum+
			     " name="+actor.getName());
	    try {
		FTAEnsembleSub es = getEnsembleById(ennum);
		localAddActor(es,actor);
	    }
	    catch(FTAException e) {
		e.printStackTrace();
	    }
	}
    }
  
    /**
     * return a reply to the calling server
     * @param req request data
     * @return reply
     */
    public FTAServletReply reply(FTAServletRequest req) {

	FTAHttpServer server = req.getServer();
	FTADaemon daemon = (FTADaemon)server.getContainer();
	String head = req.getHead();
	String tail = req.getTail();

	/* root is already: http://host:port/fta/sysname" */
	if (head.equals("/") || head.equals("/index.html"))
	    return replyTop(daemon);
	else if (head.equals("/$SYSINFO$")) {
	    req.shift();
	    return _sysinfo.reply(req);
	} else if (head.equals("/$EVALS$")) {
	    return replyEvalables(daemon);
	} else if (head.equals("/$ENM$") || head.equals("/$TBM$")) {
	    String head1 = req.shift().substring(1);
	    return replyObjs(daemon,head1,req,_localm);
	} else if (head.equals("/$ENS$") || head.equals("/$TBS$")) {
	    String head1 = req.shift().substring(1);
	    return replyObjs(daemon,head1,req,_localsByName);
	} else if (head.equals("/$LOG$"))
	    return replyLog(daemon);
	else if (head.equals("/cgi-bin") && tail != null &&
		 tail.equals("/clearlog.ftad")) {
	    _defaultlog = new StringBuffer(10000);
	    _defaultlogcount = 0;
	    return daemon.redirect("/fta/"+_mysysname+"/$LOG$/");
	} else
	    return FTAErrorReply.error404();
    }

    private synchronized FTAServletReply replyEvalables(FTADaemon daemon) {
	FTAHtmlFile f = new FTAHtmlFile("Managed Evalables");
	f.beginUL();
	FTAEvalable e = _sysevalthread.getList();
	while (e != null) {
	    f.item(); f.text(e.evelem+" ("+e.evelem.isEval()+")");
	    e = e.evnext;
	}
	f.endUL();
	return f.finish("/","Back to Top");
    }

    private FTAServletReply replyLog(FTADaemon daemon) {
	FTAHtmlFile f = new FTAHtmlFile("Default Log for FTASystem " +
					_mysysname);
	String log = _defaultlog.toString();
	if (log.length() > 0) {
	    f.beginForm(_myurl+"/cgi-bin/clearlog.ftad");
	    f.submitButton("Clear Log");
	    f.endForm();
	    f.hr();
	}
	f.beginUL();
	f.text(log);
	f.endUL();
	f.hr();
	f.beginForm(_myurl+"/cgi-bin/clearlog.ftad");
	f.submitButton("Clear Log");
	f.endForm();
	return f.finish("/fta/"+_mysysname+"/","Back to System");
    }

    private FTAServletReply replyTop(FTADaemon daemon) {
	String main = _isfirst ? " (Mainhost)" : " (Office)";
	FTAHtmlFile f = new FTAHtmlFile("FTASystem " + _mysysname +
					main);
	f.h2("Properties");
	f.beginUL();
	f.propitem("System name",_mysysname);
	f.propitem("Hostname",_myhostname+" (HostNum="+_myobjnum+")");
	f.propitem("System state",_sysstate +
		   " ( "+SysStates[_sysstate] + ')');
	f.propitem("Loop counter",_evalcnt+"");
	Date now = new Date();
	f.propitem("Time",now+"");
	if (_isfirst) {
	    f.item(); f.anchor(_myurl+"/$LOG$/index.html","Default Log");
	}
	f.endUL();
	if (_isfirst) {
	    f.beginTable();
	    f.tableNewRow("");
	    f.beginForm("/cgi-bin/controlsystem.ftad?sysname="+
			_mysysname + "&function=1");
	    f.submitButton("Run");
	    f.endForm();
	    f.tableColumn();
	    f.beginForm("/cgi-bin/controlsystem.ftad?sysname="+
			_mysysname + "&function=2");
	    f.submitButton("Stop");
	    f.endForm();
	    f.tableColumn();
	    f.beginForm("/cgi-bin/controlsystem.ftad?sysname="+
			_mysysname + "&function=3");
	    f.submitButton("Shutdown");
	    f.endForm();
	    f.endTable();

	    addSysTBLinks(f,"Hosts",
			  _sysinfo.getTable(FTASysTag.HOST),
			  "",true);
	    f.endUL();
	    addSysTBLinks(f,"Tagboards",
			  _sysinfo.getTable(FTASysTag.TB),
			  "/$TBM$/",false);
	    f.item();
	    f.anchor(FTADaemon.buildURL(_myhostname,
					"/fta/"+_mysysname+
					"/$SYSINFO$/index.html"),
		     "$SYSINFO$");
	    f.endUL();
	    addSysTBLinks(f,"Ensembles",
			  _sysinfo.getTable(FTASysTag.EN),
			  "/$ENM$/",false);
	    f.endUL();
	}

	f.h2("Managed Offices");
	addLinks(f,_localsByName);

	f.h2("Diagnostics");
	f.beginUL();
	f.item(); f.anchor(FTADaemon.buildURL(_myhostname,
					      "/fta/"+_mysysname+
					      "/$EVALS$/index.html"),
			   "Evalables");
	f.endUL();

	return f.finish("/","FTADaemon",getURL(4));
    }

    private void addSysTBLinks(FTAHtmlFile f,
			       String title,
			       Hashtable h,
			       String postfix,boolean hflag) {
	f.h2("Available "+title);
	f.beginUL();
	Enumeration it = h.elements();
	while (it.hasMoreElements()) {
	    FTASysTag tag = (FTASysTag)it.nextElement();
	    String logname = tag.getLogname();
	    if (logname.equals("$SYSINFO$"))
		continue;
	    String hostname = tag.getHostname();
	    String sysname = hflag ? "" : logname;
	    f.item();
	    f.anchor(FTADaemon.buildURL(hostname,"/fta/"+_mysysname+
					postfix+sysname+"/index.html"),
		     logname);
	    f.text(" "+tag.getObjnum());
	}
    }

    private void addLinks(FTAHtmlFile f,Hashtable h) {
	Enumeration it = h.keys();
	f.beginUL();
	while (it.hasMoreElements()) {
	    f.item();
	    String key = (String)it.nextElement();
	    FTAServletI sl = (FTAServletI)h.get(key);
	    f.anchor(sl.getURL(0),key);
	}
	f.endUL();
    }

    // already arriving here with head = name, tail = remainingpath
    private FTAServletReply replyObjs(FTADaemon daemon,String head,
				      FTAServletRequest req,
				      Hashtable h) {
	FTAServletI sl = (FTAServletI)h.get(head);
	if (sl != null) {
	    req.shift();
	    return sl.reply(req);
	}
	return FTAErrorReply.error404();
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public String getURL(int n) {
	switch (n) {
	case 4: /* its daemon */
	    return FTADaemon.buildURL(_myhostname,"/");
	case 1: /* its syssub */
	case 2: /* its sysmain */
	case 3: /* its parent */
	default: /* itself */
	    return FTADaemon.buildURL(_myhostname,"/fta/"+_mysysname+"/");
	}
    }

    public String getBaseURL(String hostname) {
	return FTADaemon.buildURL(hostname,"/fta/"+_mysysname);
    }
}

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
