// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTATagboardSub.java,v 1.15 2001/06/21 14:57:27 veit Exp $

package jfta.tagboard;

import java.net.*;
import java.util.*;

import jfta.tags.*;
import jfta.base.*;
import jfta.bootstrap.*;
import jfta.exception.*;
import jfta.system.*;
import jfta.ensemble.*;
import jfta.http.*;
import jfta.actors.*;

/**
  * TagboardSub (TS) is the repository for tags on some host. 
  * It accepts tags and getTag requests.
  * It will do a flip when instructed by a flipnow message and will
  * acknowledge successful flip with a flipdone message.
  */
public class FTATagboardSub extends Object
    implements FTAServletI,FTATagboardI,FTAEvalableI {
  
    private static FTAService _svc = FTAService.Instance(); 

    private long _cycletime;
    private long _millis;
    private FTATagHash _backside;
    private FTATagHash _frontside;
    private String _mylogname;
    private String _myhostname;
    private FTASysSub _mysyssub;
    private FTACommManager _comm;
    private int _myobjnum;
    private int _myhostnum;
    private FTAListenerI _listener;
    private boolean _flipping;

    private int _state;
    private long _precyc;
    private long _premillis;
    private Object _runlock = new Object();
    private boolean _runprep;
    private boolean _runnow;
    private boolean _runack;

    /**************************************************************/
    /* base facilities */
    /**************************************************************/
    
    /**
     * @param syssub local system office
     * @param tbname logical name of tagboard
     * @param objid objectid of this office
     * @param start start immediately
     */
    public FTATagboardSub(FTASysSub syssub,String tbname,int objnum) {
	super();
	
	_mylogname = tbname;
	_mysyssub = syssub;
	_myhostnum = syssub.getObjnum();
	_myobjnum = objnum;
	_myhostname = _svc.getFQDNname();

	_cycletime = 0;
	_millis = 0;

	// write only tb
	_backside = new FTATagHash(this,_cycletime+1);
	// read only tb
	_frontside = new FTATagHash(this,_cycletime);

	_comm = _mysyssub.getCommManager();
	_listener = new TSListener();

	// finally notify TBMain that I am operable
	_comm.sendMsg(0,FTACreateTBOfficeDoneMsg.get(_myhostnum,
						     _myobjnum));
    }
    
    /**
     */
    public String toString() {
	return "TS_"+_mylogname;
    }

    /**
     * FTAEvalable API
     * This will be called at the end of a cycle of the FTASysSub.SysEvalThread.
     * It will check whether the EnsembleSub will be rescheduled in the
     * next cycle
     */
    public synchronized int eval() throws FTAEvalException {
	synchronized(_runlock) {
	    // have to return an acknowledge
	    if (_runack) {
		/* notify flipdone back to main */
		_comm.directAckCycle(_myhostnum,_myobjnum,_cycletime,false);
	    }
	    
	    //FTADebug.log(_myobjnum,0,"Eval "+_mylogname+": p="+
	    // _runprep+" n="+_runnow+
	    // " a="+_runack);
	    
	    // update the job queue
	    _runack = _runnow;
	    _runnow = _runprep;
	    _runprep = false;
	    
	    // will we do a flip?
	    if (_runnow) {
		
		// do the flip
		FTATagHash swap = _frontside;
		_frontside = _backside;
		_backside = swap; 
		_backside.setTime(_precyc+1);
		_backside.clear();
		
		// copy the new time stamp
		_cycletime = _precyc;
		_millis = _premillis;
	    }
	}

	// will we terminate?
	if (_state==FTASysSub.FTACTLSHUTDOWN)
	    return TERMINATE;

	// we still exist
	return KEEPALIVE;
    }

    public boolean isEval() {
	return true;
    }

    /**
     * this adapter manages registered messages
     */
    private class TSListener extends FTAListenerAdapter {
	public TSListener() {
	    super();
	    _comm.addListener(FTAMsg.STARTCYCLE,this,_myobjnum);
	    _comm.addListener(FTAMsg.PUTTAG,this,_myobjnum);
	    _comm.addListener(FTAMsg.CONTROL,this);
	}

	public void uninstall(FTACommManager comm) {
	    comm.removeListener(FTAMsg.STARTCYCLE,this);
	    comm.removeListener(FTAMsg.PUTTAG,this);
	    comm.removeListener(FTAMsg.CONTROL,this);
	}
	
	// issued by SysMain to control operation
	public void notifyControl(int function, int code) {
	    if (FTADebug.CT)
		FTADebug.log(_myhostnum,_myobjnum,
			     " control: " + function);
	    _state = function;
	}

	// broadcasted by some actor via SysSub
	public void notifyPutTag(FTATag tag) {
	    if (FTADebug.TC2)
		FTADebug.log(_myhostnum,_myobjnum,
			     "tag received: "+ tag.getType());
	    // put a tag to the backside
	    tag.postInit();
	    tag.link = null;
	    _backside.put(tag);
	}

	/* issued by TBMain to its TBSubs */
	public void notifyStartCycle(long cyc,long millis) {
	    // ignore a duplicate
	    if (cyc == _cycletime)
		return;

	    if (FTADebug.FT)
		FTADebug.log(_myhostnum,_myobjnum,"flipnow: "+ cyc);

	    synchronized(_runlock) {
		/* set new time */
		_precyc = cyc;
		_premillis = millis;

		// request a run slice next time
		_runprep = true;
	    }
	}
    }

    /**************************************************************/
    /* base tagboard methods */
    /**************************************************************/

    public final long getTagboardTime(int format) {
	if (_flipping) System.out.println("TagboardTime query while flipping");
	return (format==FTAActor.ABSOLUTE) ? _millis : _cycletime;
    }
    
    public final long getTime() {
	return _cycletime;
    }

    /**
     * @roseuid 38B3E5B42080
     */
    private FTATag createTag(String type, String args[])
	throws FTATagException,FTAArgumentException {
	FTATag tag = null;
	Object tagobj = null;
	
	try {
	    Class tagclass = Class.forName(type); 
	    tagobj = tagclass.newInstance(); 
	}
	catch (Exception e) {
	    throw new FTATagException(type+".ctor() failed: "+e);
	}
	
	if (tagobj instanceof FTATag) {
	    try {
		tag = (FTATag) tagobj;
		tag.init(args);
		return tag;
	    }
	    catch(Exception e) {
		throw new FTATagException(type+".init() failed: "+e);
	    }
	} else
	    throw new FTATagException(type+".ctor() failed: invalid base class");
    }
    
    /**
     */
    public int getTagCount(String name) {
	return _frontside.getTagCount(name);
    }
    
    /**************************************************************/
    /* remote interface */
    /**************************************************************/
    
    /**
     * @roseuid 38B3E5B8D210
     */
    public FTATagIterator getTagIterator(String tagtype) {
	return _frontside.getIterator(tagtype);
    }

    /**
     */
    public String[] getTagTypes() {
	return _frontside.getTagTypes();
    }
    
    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public String getURL(int n) {
	switch (n) {
	case 4: /* its daemon */
	    return _mysyssub.getURL(4);
	case 2: /* its sysmain */
	    return _mysyssub.getURL(3);
	case 3: /* its parent */
	    return _mysyssub.getURL(1)+"$TBM$/"+_mylogname+"/";
	case 1: /* its syssub */
	    return _mysyssub.getURL(1);
	default: /* itself */
	    return _mysyssub.getURL(1)+"$TBS$/"+_mylogname+"/";
	}
    }

    /**************************************************************/
    /* HTML support */
    /**************************************************************/

    public FTAServletReply reply(FTAServletRequest req) {
	String head = req.getHead();
	if (head.equals("/") || head.equals("/index.html")) {
	    FTAHtmlFile f = new FTAHtmlFile("FTATagboard Office " + 
					    _mylogname);
	    f.h2("Properties");
	    f.beginUL();
	    f.propitem("Logical name",_mylogname);
	    f.propitem("Hostname",_myhostname+" (HostNum="+_myhostnum+")");
	    f.propitem("ObjectNum",_myobjnum+"");
	    f.propitem("Current Flip Cycle",_cycletime+"");
	    //f.propitem("Flags","a="+_runack+", n="+_runnow+", p="+_runprep);
	    Date now = new Date();
	    f.propitem("Time",now+"");
	    f.endUL();
	    f.text("The following lists are inaccurate, and links may even fail, when the system is running.");
	    f.h2("Stored tags (visible side)");
	    listTags(f,_frontside,"f/");
	    f.h2("Stored tags (back side)");
	    listTags(f,_backside,"b/");
	    return f.finish(getURL(4),"FTADaemon",getURL(2));
	} else {
	    // expect /f or /b here
	    FTATagHash th = null;
	    if (head.equals("/f")) th = _frontside;
	    else if (head.equals("/b")) th = _backside;
	    if (th != null) {
		try {
		    head = req.shift().substring(1);
		    String head2 = req.shift().substring(1);
		    int hash = Integer.parseInt(head2);
		    FTATag tag = th.getTagByHash(head,hash);
		    if (tag != null) {
			req.shift();
			return tag.reply(req);
		    }
		}
		catch(NumberFormatException e) {
		    // will fall through to error404
		}
	    }
	    return FTAErrorReply.error404();
	} 
    }

    private void listTags(FTAHtmlFile f,FTATagHash th,String pfx) {

	Enumeration tsi = th.getStoreIterator();
	String myurl = getURL(0);
	f.beginOL();
	while(tsi.hasMoreElements()) {
	    String tagtype = (String)tsi.nextElement();
	    FTATagIterator ti = th.getIterator(tagtype);
	    while (ti.hasNext()) {
		Object tg = ti.next2();
		f.item(); f.anchor(myurl+pfx+tagtype+"/"+
				   tg.hashCode()+"/",
				   tg.toString());
	    }
	}
	f.endOL();

    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
