// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTATagIterator.java,v 1.4 2001/06/21 14:57:27 veit Exp $

package jfta.tags;

import java.util.*;
import jfta.exception.*;
import jfta.tagboard.*;

/**
 * The base class for all iterators that operate on Tags.
 * @author Holger Veit, Elmar Degen
 * @version $Id: FTATagIterator.java,v 1.4 2001/06/21 14:57:27 veit Exp $
 * @since JFTA1.0
 */
public class FTATagIterator {
    protected long cycletime;
    protected FTATagboardI tagboard;
    
    /**
     * check whether there is another element that matches the criteria
     * @return true, if there is one, false if not
     * @roseuid 380B31AF5950
     */
    public boolean hasNext() {
	return false;
    }
    
    /**
     * return the next element
     * @return the next element that matches the criteria
     * @exception FTATagboardFlipException thrown if an asynchronous
     *   event happens (a flip of the tagboard) or the iterator is
     *   at its end.
     * @roseuid 380B31AF5951
     */
    public FTATag next() throws FTATagboardFlipException {
	return next2();
    }

    /**
     * return the next element, but does not throw exception
     * @return the next element that matches the criteria
     */
    public FTATag next2() {
	return null;
    }


    
    /**
     * return the current time (flip count) of the tagboard that the iterator 
     * uses to check whether it is still valid
     * @return the current flip time that the iterator expects to be valid
     */
    public long getIteratorTime() {
	return cycletime;
    }
    
    /**
     * Constructor. This is used internally and cannot be used
     * to produce a valid iterator.
     * @roseuid 380B39C3B3D8
     */
    public FTATagIterator(FTATagboardI ts) {
	super();
	tagboard = ts;
	cycletime = ts.getTime();
    }

    /**
     * return true if the tagboard has flipped in the meantime
     * @return true: tagboard has flipped
     */
    protected boolean hasFlipped() { 
	return tagboard.getTime() != cycletime;
    }
    
    public FTATagIterator() {}
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
